from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.column_pii_contains_email_percent_check_spec import (
        ColumnPiiContainsEmailPercentCheckSpec,
    )
    from ..models.column_pii_contains_ip_4_percent_check_spec import (
        ColumnPiiContainsIp4PercentCheckSpec,
    )
    from ..models.column_pii_contains_ip_6_percent_check_spec import (
        ColumnPiiContainsIp6PercentCheckSpec,
    )
    from ..models.column_pii_contains_usa_phone_percent_check_spec import (
        ColumnPiiContainsUsaPhonePercentCheckSpec,
    )
    from ..models.column_pii_contains_usa_zipcode_percent_check_spec import (
        ColumnPiiContainsUsaZipcodePercentCheckSpec,
    )
    from ..models.column_pii_profiling_checks_spec_custom_checks import (
        ColumnPiiProfilingChecksSpecCustomChecks,
    )


T = TypeVar("T", bound="ColumnPiiProfilingChecksSpec")


@_attrs_define
class ColumnPiiProfilingChecksSpec:
    """
    Attributes:
        custom_checks (Union[Unset, ColumnPiiProfilingChecksSpecCustomChecks]): Dictionary of additional custom checks
            within this category. The keys are check names defined in the definition section. The sensor parameters and
            rules should match the type of the configured sensor and rule for the custom check.
        profile_contains_usa_phone_percent (Union[Unset, ColumnPiiContainsUsaPhonePercentCheckSpec]):
        profile_contains_email_percent (Union[Unset, ColumnPiiContainsEmailPercentCheckSpec]):
        profile_contains_usa_zipcode_percent (Union[Unset, ColumnPiiContainsUsaZipcodePercentCheckSpec]):
        profile_contains_ip4_percent (Union[Unset, ColumnPiiContainsIp4PercentCheckSpec]):
        profile_contains_ip6_percent (Union[Unset, ColumnPiiContainsIp6PercentCheckSpec]):
    """

    custom_checks: Union[Unset, "ColumnPiiProfilingChecksSpecCustomChecks"] = UNSET
    profile_contains_usa_phone_percent: Union[
        Unset, "ColumnPiiContainsUsaPhonePercentCheckSpec"
    ] = UNSET
    profile_contains_email_percent: Union[
        Unset, "ColumnPiiContainsEmailPercentCheckSpec"
    ] = UNSET
    profile_contains_usa_zipcode_percent: Union[
        Unset, "ColumnPiiContainsUsaZipcodePercentCheckSpec"
    ] = UNSET
    profile_contains_ip4_percent: Union[
        Unset, "ColumnPiiContainsIp4PercentCheckSpec"
    ] = UNSET
    profile_contains_ip6_percent: Union[
        Unset, "ColumnPiiContainsIp6PercentCheckSpec"
    ] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        custom_checks: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.custom_checks, Unset):
            custom_checks = self.custom_checks.to_dict()

        profile_contains_usa_phone_percent: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.profile_contains_usa_phone_percent, Unset):
            profile_contains_usa_phone_percent = (
                self.profile_contains_usa_phone_percent.to_dict()
            )

        profile_contains_email_percent: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.profile_contains_email_percent, Unset):
            profile_contains_email_percent = (
                self.profile_contains_email_percent.to_dict()
            )

        profile_contains_usa_zipcode_percent: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.profile_contains_usa_zipcode_percent, Unset):
            profile_contains_usa_zipcode_percent = (
                self.profile_contains_usa_zipcode_percent.to_dict()
            )

        profile_contains_ip4_percent: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.profile_contains_ip4_percent, Unset):
            profile_contains_ip4_percent = self.profile_contains_ip4_percent.to_dict()

        profile_contains_ip6_percent: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.profile_contains_ip6_percent, Unset):
            profile_contains_ip6_percent = self.profile_contains_ip6_percent.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if custom_checks is not UNSET:
            field_dict["custom_checks"] = custom_checks
        if profile_contains_usa_phone_percent is not UNSET:
            field_dict["profile_contains_usa_phone_percent"] = (
                profile_contains_usa_phone_percent
            )
        if profile_contains_email_percent is not UNSET:
            field_dict["profile_contains_email_percent"] = (
                profile_contains_email_percent
            )
        if profile_contains_usa_zipcode_percent is not UNSET:
            field_dict["profile_contains_usa_zipcode_percent"] = (
                profile_contains_usa_zipcode_percent
            )
        if profile_contains_ip4_percent is not UNSET:
            field_dict["profile_contains_ip4_percent"] = profile_contains_ip4_percent
        if profile_contains_ip6_percent is not UNSET:
            field_dict["profile_contains_ip6_percent"] = profile_contains_ip6_percent

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.column_pii_contains_email_percent_check_spec import (
            ColumnPiiContainsEmailPercentCheckSpec,
        )
        from ..models.column_pii_contains_ip_4_percent_check_spec import (
            ColumnPiiContainsIp4PercentCheckSpec,
        )
        from ..models.column_pii_contains_ip_6_percent_check_spec import (
            ColumnPiiContainsIp6PercentCheckSpec,
        )
        from ..models.column_pii_contains_usa_phone_percent_check_spec import (
            ColumnPiiContainsUsaPhonePercentCheckSpec,
        )
        from ..models.column_pii_contains_usa_zipcode_percent_check_spec import (
            ColumnPiiContainsUsaZipcodePercentCheckSpec,
        )
        from ..models.column_pii_profiling_checks_spec_custom_checks import (
            ColumnPiiProfilingChecksSpecCustomChecks,
        )

        d = src_dict.copy()
        _custom_checks = d.pop("custom_checks", UNSET)
        custom_checks: Union[Unset, ColumnPiiProfilingChecksSpecCustomChecks]
        if isinstance(_custom_checks, Unset):
            custom_checks = UNSET
        else:
            custom_checks = ColumnPiiProfilingChecksSpecCustomChecks.from_dict(
                _custom_checks
            )

        _profile_contains_usa_phone_percent = d.pop(
            "profile_contains_usa_phone_percent", UNSET
        )
        profile_contains_usa_phone_percent: Union[
            Unset, ColumnPiiContainsUsaPhonePercentCheckSpec
        ]
        if isinstance(_profile_contains_usa_phone_percent, Unset):
            profile_contains_usa_phone_percent = UNSET
        else:
            profile_contains_usa_phone_percent = (
                ColumnPiiContainsUsaPhonePercentCheckSpec.from_dict(
                    _profile_contains_usa_phone_percent
                )
            )

        _profile_contains_email_percent = d.pop("profile_contains_email_percent", UNSET)
        profile_contains_email_percent: Union[
            Unset, ColumnPiiContainsEmailPercentCheckSpec
        ]
        if isinstance(_profile_contains_email_percent, Unset):
            profile_contains_email_percent = UNSET
        else:
            profile_contains_email_percent = (
                ColumnPiiContainsEmailPercentCheckSpec.from_dict(
                    _profile_contains_email_percent
                )
            )

        _profile_contains_usa_zipcode_percent = d.pop(
            "profile_contains_usa_zipcode_percent", UNSET
        )
        profile_contains_usa_zipcode_percent: Union[
            Unset, ColumnPiiContainsUsaZipcodePercentCheckSpec
        ]
        if isinstance(_profile_contains_usa_zipcode_percent, Unset):
            profile_contains_usa_zipcode_percent = UNSET
        else:
            profile_contains_usa_zipcode_percent = (
                ColumnPiiContainsUsaZipcodePercentCheckSpec.from_dict(
                    _profile_contains_usa_zipcode_percent
                )
            )

        _profile_contains_ip4_percent = d.pop("profile_contains_ip4_percent", UNSET)
        profile_contains_ip4_percent: Union[Unset, ColumnPiiContainsIp4PercentCheckSpec]
        if isinstance(_profile_contains_ip4_percent, Unset):
            profile_contains_ip4_percent = UNSET
        else:
            profile_contains_ip4_percent = (
                ColumnPiiContainsIp4PercentCheckSpec.from_dict(
                    _profile_contains_ip4_percent
                )
            )

        _profile_contains_ip6_percent = d.pop("profile_contains_ip6_percent", UNSET)
        profile_contains_ip6_percent: Union[Unset, ColumnPiiContainsIp6PercentCheckSpec]
        if isinstance(_profile_contains_ip6_percent, Unset):
            profile_contains_ip6_percent = UNSET
        else:
            profile_contains_ip6_percent = (
                ColumnPiiContainsIp6PercentCheckSpec.from_dict(
                    _profile_contains_ip6_percent
                )
            )

        column_pii_profiling_checks_spec = cls(
            custom_checks=custom_checks,
            profile_contains_usa_phone_percent=profile_contains_usa_phone_percent,
            profile_contains_email_percent=profile_contains_email_percent,
            profile_contains_usa_zipcode_percent=profile_contains_usa_zipcode_percent,
            profile_contains_ip4_percent=profile_contains_ip4_percent,
            profile_contains_ip6_percent=profile_contains_ip6_percent,
        )

        column_pii_profiling_checks_spec.additional_properties = d
        return column_pii_profiling_checks_spec

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.column_schema_column_exists_check_spec import (
        ColumnSchemaColumnExistsCheckSpec,
    )
    from ..models.column_schema_profiling_checks_spec_custom_checks import (
        ColumnSchemaProfilingChecksSpecCustomChecks,
    )
    from ..models.column_schema_type_changed_check_spec import (
        ColumnSchemaTypeChangedCheckSpec,
    )


T = TypeVar("T", bound="ColumnSchemaProfilingChecksSpec")


@_attrs_define
class ColumnSchemaProfilingChecksSpec:
    """
    Attributes:
        custom_checks (Union[Unset, ColumnSchemaProfilingChecksSpecCustomChecks]): Dictionary of additional custom
            checks within this category. The keys are check names defined in the definition section. The sensor parameters
            and rules should match the type of the configured sensor and rule for the custom check.
        profile_column_exists (Union[Unset, ColumnSchemaColumnExistsCheckSpec]):
        profile_column_type_changed (Union[Unset, ColumnSchemaTypeChangedCheckSpec]):
    """

    custom_checks: Union[Unset, "ColumnSchemaProfilingChecksSpecCustomChecks"] = UNSET
    profile_column_exists: Union[Unset, "ColumnSchemaColumnExistsCheckSpec"] = UNSET
    profile_column_type_changed: Union[Unset, "ColumnSchemaTypeChangedCheckSpec"] = (
        UNSET
    )
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        custom_checks: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.custom_checks, Unset):
            custom_checks = self.custom_checks.to_dict()

        profile_column_exists: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.profile_column_exists, Unset):
            profile_column_exists = self.profile_column_exists.to_dict()

        profile_column_type_changed: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.profile_column_type_changed, Unset):
            profile_column_type_changed = self.profile_column_type_changed.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if custom_checks is not UNSET:
            field_dict["custom_checks"] = custom_checks
        if profile_column_exists is not UNSET:
            field_dict["profile_column_exists"] = profile_column_exists
        if profile_column_type_changed is not UNSET:
            field_dict["profile_column_type_changed"] = profile_column_type_changed

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.column_schema_column_exists_check_spec import (
            ColumnSchemaColumnExistsCheckSpec,
        )
        from ..models.column_schema_profiling_checks_spec_custom_checks import (
            ColumnSchemaProfilingChecksSpecCustomChecks,
        )
        from ..models.column_schema_type_changed_check_spec import (
            ColumnSchemaTypeChangedCheckSpec,
        )

        d = src_dict.copy()
        _custom_checks = d.pop("custom_checks", UNSET)
        custom_checks: Union[Unset, ColumnSchemaProfilingChecksSpecCustomChecks]
        if isinstance(_custom_checks, Unset):
            custom_checks = UNSET
        else:
            custom_checks = ColumnSchemaProfilingChecksSpecCustomChecks.from_dict(
                _custom_checks
            )

        _profile_column_exists = d.pop("profile_column_exists", UNSET)
        profile_column_exists: Union[Unset, ColumnSchemaColumnExistsCheckSpec]
        if isinstance(_profile_column_exists, Unset):
            profile_column_exists = UNSET
        else:
            profile_column_exists = ColumnSchemaColumnExistsCheckSpec.from_dict(
                _profile_column_exists
            )

        _profile_column_type_changed = d.pop("profile_column_type_changed", UNSET)
        profile_column_type_changed: Union[Unset, ColumnSchemaTypeChangedCheckSpec]
        if isinstance(_profile_column_type_changed, Unset):
            profile_column_type_changed = UNSET
        else:
            profile_column_type_changed = ColumnSchemaTypeChangedCheckSpec.from_dict(
                _profile_column_type_changed
            )

        column_schema_profiling_checks_spec = cls(
            custom_checks=custom_checks,
            profile_column_exists=profile_column_exists,
            profile_column_type_changed=profile_column_type_changed,
        )

        column_schema_profiling_checks_spec.additional_properties = d
        return column_schema_profiling_checks_spec

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class DataTypeCategory(str, Enum):
    ARRAY = "array"
    BINARY = "binary"
    BOOL = "bool"
    CLOB = "clob"
    DATETIME_DATE = "datetime_date"
    DATETIME_DATETIME = "datetime_datetime"
    DATETIME_TIME = "datetime_time"
    DATETIME_TIMESTAMP = "datetime_timestamp"
    JSON = "json"
    NUMERIC_DECIMAL = "numeric_decimal"
    NUMERIC_FLOAT = "numeric_float"
    NUMERIC_INTEGER = "numeric_integer"
    OTHER = "other"
    TEXT = "text"

    def __str__(self) -> str:
        return str(self.value)



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.check_time_scale import CheckTimeScale
from ...models.table_comparison_results_model import TableComparisonResultsModel
from ...types import Response


def _get_kwargs(
    connection_name: str,
    schema_name: str,
    table_name: str,
    time_scale: CheckTimeScale,
    table_comparison_configuration_name: str,
) -> Dict[str, Any]:

    pass

    return {
        "method": "get",
        "url": "api/connections/{connectionName}/schemas/{schemaName}/tables/{tableName}/partitioned/{timeScale}/comparisons/{tableComparisonConfigurationName}/results".format(
            connectionName=connection_name,
            schemaName=schema_name,
            tableName=table_name,
            timeScale=time_scale,
            tableComparisonConfigurationName=table_comparison_configuration_name,
        ),
    }


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[TableComparisonResultsModel]:
    if response.status_code == HTTPStatus.OK:
        response_200 = TableComparisonResultsModel.from_dict(response.json())

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[TableComparisonResultsModel]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    connection_name: str,
    schema_name: str,
    table_name: str,
    time_scale: CheckTimeScale,
    table_comparison_configuration_name: str,
    *,
    client: AuthenticatedClient,
) -> Response[TableComparisonResultsModel]:
    """getTableComparisonPartitionedResults

     Retrieves the results of the most table comparison performed using the partitioned comparison
    checks, comparing days or months of data.

    Args:
        connection_name (str):
        schema_name (str):
        table_name (str):
        time_scale (CheckTimeScale):
        table_comparison_configuration_name (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[TableComparisonResultsModel]
    """

    kwargs = _get_kwargs(
        connection_name=connection_name,
        schema_name=schema_name,
        table_name=table_name,
        time_scale=time_scale,
        table_comparison_configuration_name=table_comparison_configuration_name,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    connection_name: str,
    schema_name: str,
    table_name: str,
    time_scale: CheckTimeScale,
    table_comparison_configuration_name: str,
    *,
    client: AuthenticatedClient,
) -> Optional[TableComparisonResultsModel]:
    """getTableComparisonPartitionedResults

     Retrieves the results of the most table comparison performed using the partitioned comparison
    checks, comparing days or months of data.

    Args:
        connection_name (str):
        schema_name (str):
        table_name (str):
        time_scale (CheckTimeScale):
        table_comparison_configuration_name (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        TableComparisonResultsModel
    """

    return sync_detailed(
        connection_name=connection_name,
        schema_name=schema_name,
        table_name=table_name,
        time_scale=time_scale,
        table_comparison_configuration_name=table_comparison_configuration_name,
        client=client,
    ).parsed


async def asyncio_detailed(
    connection_name: str,
    schema_name: str,
    table_name: str,
    time_scale: CheckTimeScale,
    table_comparison_configuration_name: str,
    *,
    client: AuthenticatedClient,
) -> Response[TableComparisonResultsModel]:
    """getTableComparisonPartitionedResults

     Retrieves the results of the most table comparison performed using the partitioned comparison
    checks, comparing days or months of data.

    Args:
        connection_name (str):
        schema_name (str):
        table_name (str):
        time_scale (CheckTimeScale):
        table_comparison_configuration_name (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[TableComparisonResultsModel]
    """

    kwargs = _get_kwargs(
        connection_name=connection_name,
        schema_name=schema_name,
        table_name=table_name,
        time_scale=time_scale,
        table_comparison_configuration_name=table_comparison_configuration_name,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    connection_name: str,
    schema_name: str,
    table_name: str,
    time_scale: CheckTimeScale,
    table_comparison_configuration_name: str,
    *,
    client: AuthenticatedClient,
) -> Optional[TableComparisonResultsModel]:
    """getTableComparisonPartitionedResults

     Retrieves the results of the most table comparison performed using the partitioned comparison
    checks, comparing days or months of data.

    Args:
        connection_name (str):
        schema_name (str):
        table_name (str):
        time_scale (CheckTimeScale):
        table_comparison_configuration_name (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        TableComparisonResultsModel
    """

    return (
        await asyncio_detailed(
            connection_name=connection_name,
            schema_name=schema_name,
            table_name=table_name,
            time_scale=time_scale,
            table_comparison_configuration_name=table_comparison_configuration_name,
            client=client,
        )
    ).parsed



from enum import Enum


class RedshiftAuthenticationMode(str, Enum):
    DEFAULT_CREDENTIALS = "default_credentials"
    IAM = "iam"
    USER_PASSWORD = "user_password"

    def __str__(self) -> str:
        return str(self.value)



#!/usr/bin/env python3

#
# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from glob import glob
import os
import sys
import zipfile
import jdk
from setuptools import setup
from setuptools import find_packages
import importlib.util
from setuptools.command.install import install
from shutil import rmtree

# ignore those, they are filled by importing dqops/version.py
VERSION = "filled by dqops/version.py"
PIP_VERSION = "filled by dqops/version.py"
GITHUB_RELEASE = "filled by dqops/version.py"
JAVA_VERSION = "filled by dqops/version.py"


HOME_PATH = "dqops/home"
JRE_PATH = "dqops/jre"
DQO_REPO_HOME = os.path.abspath("../../")

try:
    exec(open('dqops/version.py').read())
except IOError:
    print("Failed to load DQOps version file for packaging. You must be in DQOps distribution/python dir.",
          file=sys.stderr)
    sys.exit(-1)

try:
    download_module_spec = importlib.util.spec_from_file_location("install", "dqops/install.py")
    download_module = importlib.util.module_from_spec(download_module_spec)
    download_module_spec.loader.exec_module(download_module)
except IOError:
    print("Failed to load the download module (dqops/install.py) which had to be packaged together.",
          file=sys.stderr)
    sys.exit(-1)

# Provide guidance about how to use setup.py
incorrect_invocation_message = """
If you are installing DQOps from https://github.com/dqops/dqo source, you must first build DQOps.

    To build DQOps with maven, go to the https://github.com/dqops/dqo root folder and run run:
      ./mvnw -DskipTests clean package"""

# Figure out where the distribution file is present
DISTRIBUTION_PATH = os.path.join(DQO_REPO_HOME, "distribution/target/dqo-distribution-{0}-bin.zip".format(VERSION))

in_dqo = os.path.isfile(os.path.join(DQO_REPO_HOME, "pom.xml")) and \
         os.path.isfile(os.path.join(DQO_REPO_HOME, "dqops/src/main/java/com/dqops/cli/CliApplication.java"))


class InstallCommand(install):
    def run(self):
        install.run(self)

        # download and install DQO
        if not in_dqo:
            download_module.install_dqo(HOME_PATH, GITHUB_RELEASE, VERSION)

        os.makedirs(JRE_PATH, exist_ok=True)

        # install a local JRE
        print("Installing Java JRE %s at %s" % (JAVA_VERSION, JRE_PATH))
        jdk.install(JAVA_VERSION, jre=True, path=JRE_PATH)

try:
    if in_dqo:
        # Create a folder to expand the distribution
        try:
            os.mkdir(HOME_PATH)
        except:
            print("Cannot create {0}".format(HOME_PATH), file=sys.stderr)
            sys.exit(-1)

        if not os.path.isfile(DISTRIBUTION_PATH) and not os.path.exists(HOME_PATH):
            print(incorrect_invocation_message, file=sys.stderr)
            sys.exit(-1)

        with zipfile.ZipFile(DISTRIBUTION_PATH, "r") as zip_ref:
            zip_ref.extractall(HOME_PATH)

    with open('README.md') as f:
        long_description = f.read()

#    home_dirs = [root[len("dqops/"):].replace('\\', '/') + "/*" for (root, dirs, files) in os.walk(HOME_PATH)]

    setup(
        name='dqops',
        version=PIP_VERSION,
        description='DQOps Data Quality Operations Center',
        long_description=long_description,
        long_description_content_type="text/markdown",
        author='DQOps Developers',
        author_email='support@dqops.com',
        url='https://github.com/dqops/dqo/tree/master/distribution/python',
        packages=find_packages(),
        include_package_data=True,
        package_data={
            'dqops': ['py.typed']
        },
        license='http://www.apache.org/licenses/LICENSE-2.0',
        install_requires=['install-jdk>=1.1.0',
                          'h2>=4.0.0',
                          'httpx[http2]>=0.27.0',
                          'attrs>=21.3.0',
                          'python-dateutil>=2.8.2',
                          'numpy>=1.24.3',
                          'MarkupSafe>=2.1.1',
                          'Jinja2>=3.0.3',
                          'six>=1.16.0',
                          'pandas>=2.0.1',
                          'scipy>=1.10.1'],
        python_requires='>=3.8',
        classifiers=[
            'Development Status :: 5 - Production/Stable',
            'License :: OSI Approved :: Apache Software License',
            'Programming Language :: Python :: 3.8',
            'Programming Language :: Python :: 3.9',
            'Programming Language :: Python :: 3.10',
            'Programming Language :: Python :: 3.11',
            'Programming Language :: Python :: 3.12',
            'Environment :: Console',
            'Framework :: Apache Airflow :: Provider',
            'Intended Audience :: Developers',
            'Operating System :: Microsoft :: Windows',
            'Operating System :: MacOS',
            'Operating System :: POSIX :: Linux',
            'Programming Language :: Java',
            'Programming Language :: SQL',
            'Topic :: Database',
            'Topic :: Scientific/Engineering :: Artificial Intelligence',
            'Topic :: Scientific/Engineering :: Information Analysis',
            'Topic :: Security',
            'Topic :: Software Development :: Quality Assurance',
            'Topic :: Software Development :: Testing',
            'Topic :: Software Development :: Version Control :: Git',
            'Programming Language :: Python :: Implementation :: CPython',
            'Programming Language :: Python :: Implementation :: PyPy'],
        # cmdclass={
        #     'install': InstallCommand,
        # },
        entry_points={
            'console_scripts': ['dqo=dqops.startdqo:main'],
        },
    )
finally:
    if in_dqo:
        rmtree(HOME_PATH)



from enum import Enum


class CheckResultSortOrder(str, Enum):
    ACTUALVALUE = "actualValue"
    CHECKCATEGORY = "checkCategory"
    CHECKDISPLAYNAME = "checkDisplayName"
    CHECKHASH = "checkHash"
    CHECKNAME = "checkName"
    CHECKTYPE = "checkType"
    COLUMNNAME = "columnName"
    DATAGROUP = "dataGroup"
    EXECUTEDAT = "executedAt"
    EXPECTEDVALUE = "expectedValue"
    QUALITYDIMENSION = "qualityDimension"
    SENSORNAME = "sensorName"
    SEVERITY = "severity"
    TIMEGRADIENT = "timeGradient"
    TIMEPERIOD = "timePeriod"
    UPDATEDAT = "updatedAt"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class BigQueryAuthenticationMode(str, Enum):
    GOOGLE_APPLICATION_CREDENTIALS = "google_application_credentials"
    JSON_KEY_CONTENT = "json_key_content"
    JSON_KEY_PATH = "json_key_path"

    def __str__(self) -> str:
        return str(self.value)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.column_range_max_value_statistics_collector_spec import (
        ColumnRangeMaxValueStatisticsCollectorSpec,
    )
    from ..models.column_range_mean_value_statistics_collector_spec import (
        ColumnRangeMeanValueStatisticsCollectorSpec,
    )
    from ..models.column_range_median_value_statistics_collector_spec import (
        ColumnRangeMedianValueStatisticsCollectorSpec,
    )
    from ..models.column_range_min_value_statistics_collector_spec import (
        ColumnRangeMinValueStatisticsCollectorSpec,
    )
    from ..models.column_range_sum_value_statistics_collector_spec import (
        ColumnRangeSumValueStatisticsCollectorSpec,
    )


T = TypeVar("T", bound="ColumnRangeStatisticsCollectorsSpec")


@_attrs_define
class ColumnRangeStatisticsCollectorsSpec:
    """
    Attributes:
        min_value (Union[Unset, ColumnRangeMinValueStatisticsCollectorSpec]):
        median_value (Union[Unset, ColumnRangeMedianValueStatisticsCollectorSpec]):
        max_value (Union[Unset, ColumnRangeMaxValueStatisticsCollectorSpec]):
        mean_value (Union[Unset, ColumnRangeMeanValueStatisticsCollectorSpec]):
        sum_value (Union[Unset, ColumnRangeSumValueStatisticsCollectorSpec]):
    """

    min_value: Union[Unset, "ColumnRangeMinValueStatisticsCollectorSpec"] = UNSET
    median_value: Union[Unset, "ColumnRangeMedianValueStatisticsCollectorSpec"] = UNSET
    max_value: Union[Unset, "ColumnRangeMaxValueStatisticsCollectorSpec"] = UNSET
    mean_value: Union[Unset, "ColumnRangeMeanValueStatisticsCollectorSpec"] = UNSET
    sum_value: Union[Unset, "ColumnRangeSumValueStatisticsCollectorSpec"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        min_value: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.min_value, Unset):
            min_value = self.min_value.to_dict()

        median_value: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.median_value, Unset):
            median_value = self.median_value.to_dict()

        max_value: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.max_value, Unset):
            max_value = self.max_value.to_dict()

        mean_value: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.mean_value, Unset):
            mean_value = self.mean_value.to_dict()

        sum_value: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.sum_value, Unset):
            sum_value = self.sum_value.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if min_value is not UNSET:
            field_dict["min_value"] = min_value
        if median_value is not UNSET:
            field_dict["median_value"] = median_value
        if max_value is not UNSET:
            field_dict["max_value"] = max_value
        if mean_value is not UNSET:
            field_dict["mean_value"] = mean_value
        if sum_value is not UNSET:
            field_dict["sum_value"] = sum_value

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.column_range_max_value_statistics_collector_spec import (
            ColumnRangeMaxValueStatisticsCollectorSpec,
        )
        from ..models.column_range_mean_value_statistics_collector_spec import (
            ColumnRangeMeanValueStatisticsCollectorSpec,
        )
        from ..models.column_range_median_value_statistics_collector_spec import (
            ColumnRangeMedianValueStatisticsCollectorSpec,
        )
        from ..models.column_range_min_value_statistics_collector_spec import (
            ColumnRangeMinValueStatisticsCollectorSpec,
        )
        from ..models.column_range_sum_value_statistics_collector_spec import (
            ColumnRangeSumValueStatisticsCollectorSpec,
        )

        d = src_dict.copy()
        _min_value = d.pop("min_value", UNSET)
        min_value: Union[Unset, ColumnRangeMinValueStatisticsCollectorSpec]
        if isinstance(_min_value, Unset):
            min_value = UNSET
        else:
            min_value = ColumnRangeMinValueStatisticsCollectorSpec.from_dict(_min_value)

        _median_value = d.pop("median_value", UNSET)
        median_value: Union[Unset, ColumnRangeMedianValueStatisticsCollectorSpec]
        if isinstance(_median_value, Unset):
            median_value = UNSET
        else:
            median_value = ColumnRangeMedianValueStatisticsCollectorSpec.from_dict(
                _median_value
            )

        _max_value = d.pop("max_value", UNSET)
        max_value: Union[Unset, ColumnRangeMaxValueStatisticsCollectorSpec]
        if isinstance(_max_value, Unset):
            max_value = UNSET
        else:
            max_value = ColumnRangeMaxValueStatisticsCollectorSpec.from_dict(_max_value)

        _mean_value = d.pop("mean_value", UNSET)
        mean_value: Union[Unset, ColumnRangeMeanValueStatisticsCollectorSpec]
        if isinstance(_mean_value, Unset):
            mean_value = UNSET
        else:
            mean_value = ColumnRangeMeanValueStatisticsCollectorSpec.from_dict(
                _mean_value
            )

        _sum_value = d.pop("sum_value", UNSET)
        sum_value: Union[Unset, ColumnRangeSumValueStatisticsCollectorSpec]
        if isinstance(_sum_value, Unset):
            sum_value = UNSET
        else:
            sum_value = ColumnRangeSumValueStatisticsCollectorSpec.from_dict(_sum_value)

        column_range_statistics_collectors_spec = cls(
            min_value=min_value,
            median_value=median_value,
            max_value=max_value,
            mean_value=mean_value,
            sum_value=sum_value,
        )

        column_range_statistics_collectors_spec.additional_properties = d
        return column_range_statistics_collectors_spec

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.column_distinct_count_anomaly_differencing_check_spec import (
        ColumnDistinctCountAnomalyDifferencingCheckSpec,
    )
    from ..models.column_distinct_count_change_1_day_check_spec import (
        ColumnDistinctCountChange1DayCheckSpec,
    )
    from ..models.column_distinct_count_change_7_days_check_spec import (
        ColumnDistinctCountChange7DaysCheckSpec,
    )
    from ..models.column_distinct_count_change_30_days_check_spec import (
        ColumnDistinctCountChange30DaysCheckSpec,
    )
    from ..models.column_distinct_count_change_check_spec import (
        ColumnDistinctCountChangeCheckSpec,
    )
    from ..models.column_distinct_count_check_spec import ColumnDistinctCountCheckSpec
    from ..models.column_distinct_percent_anomaly_stationary_check_spec import (
        ColumnDistinctPercentAnomalyStationaryCheckSpec,
    )
    from ..models.column_distinct_percent_change_1_day_check_spec import (
        ColumnDistinctPercentChange1DayCheckSpec,
    )
    from ..models.column_distinct_percent_change_7_days_check_spec import (
        ColumnDistinctPercentChange7DaysCheckSpec,
    )
    from ..models.column_distinct_percent_change_30_days_check_spec import (
        ColumnDistinctPercentChange30DaysCheckSpec,
    )
    from ..models.column_distinct_percent_change_check_spec import (
        ColumnDistinctPercentChangeCheckSpec,
    )
    from ..models.column_distinct_percent_check_spec import (
        ColumnDistinctPercentCheckSpec,
    )
    from ..models.column_duplicate_count_check_spec import ColumnDuplicateCountCheckSpec
    from ..models.column_duplicate_percent_check_spec import (
        ColumnDuplicatePercentCheckSpec,
    )
    from ..models.column_uniqueness_profiling_checks_spec_custom_checks import (
        ColumnUniquenessProfilingChecksSpecCustomChecks,
    )


T = TypeVar("T", bound="ColumnUniquenessProfilingChecksSpec")


@_attrs_define
class ColumnUniquenessProfilingChecksSpec:
    """
    Attributes:
        custom_checks (Union[Unset, ColumnUniquenessProfilingChecksSpecCustomChecks]): Dictionary of additional custom
            checks within this category. The keys are check names defined in the definition section. The sensor parameters
            and rules should match the type of the configured sensor and rule for the custom check.
        profile_distinct_count (Union[Unset, ColumnDistinctCountCheckSpec]):
        profile_distinct_percent (Union[Unset, ColumnDistinctPercentCheckSpec]):
        profile_duplicate_count (Union[Unset, ColumnDuplicateCountCheckSpec]):
        profile_duplicate_percent (Union[Unset, ColumnDuplicatePercentCheckSpec]):
        profile_distinct_count_anomaly (Union[Unset, ColumnDistinctCountAnomalyDifferencingCheckSpec]):
        profile_distinct_percent_anomaly (Union[Unset, ColumnDistinctPercentAnomalyStationaryCheckSpec]):
        profile_distinct_count_change (Union[Unset, ColumnDistinctCountChangeCheckSpec]):
        profile_distinct_percent_change (Union[Unset, ColumnDistinctPercentChangeCheckSpec]):
        profile_distinct_count_change_1_day (Union[Unset, ColumnDistinctCountChange1DayCheckSpec]):
        profile_distinct_count_change_7_days (Union[Unset, ColumnDistinctCountChange7DaysCheckSpec]):
        profile_distinct_count_change_30_days (Union[Unset, ColumnDistinctCountChange30DaysCheckSpec]):
        profile_distinct_percent_change_1_day (Union[Unset, ColumnDistinctPercentChange1DayCheckSpec]):
        profile_distinct_percent_change_7_days (Union[Unset, ColumnDistinctPercentChange7DaysCheckSpec]):
        profile_distinct_percent_change_30_days (Union[Unset, ColumnDistinctPercentChange30DaysCheckSpec]):
    """

    custom_checks: Union[Unset, "ColumnUniquenessProfilingChecksSpecCustomChecks"] = (
        UNSET
    )
    profile_distinct_count: Union[Unset, "ColumnDistinctCountCheckSpec"] = UNSET
    profile_distinct_percent: Union[Unset, "ColumnDistinctPercentCheckSpec"] = UNSET
    profile_duplicate_count: Union[Unset, "ColumnDuplicateCountCheckSpec"] = UNSET
    profile_duplicate_percent: Union[Unset, "ColumnDuplicatePercentCheckSpec"] = UNSET
    profile_distinct_count_anomaly: Union[
        Unset, "ColumnDistinctCountAnomalyDifferencingCheckSpec"
    ] = UNSET
    profile_distinct_percent_anomaly: Union[
        Unset, "ColumnDistinctPercentAnomalyStationaryCheckSpec"
    ] = UNSET
    profile_distinct_count_change: Union[
        Unset, "ColumnDistinctCountChangeCheckSpec"
    ] = UNSET
    profile_distinct_percent_change: Union[
        Unset, "ColumnDistinctPercentChangeCheckSpec"
    ] = UNSET
    profile_distinct_count_change_1_day: Union[
        Unset, "ColumnDistinctCountChange1DayCheckSpec"
    ] = UNSET
    profile_distinct_count_change_7_days: Union[
        Unset, "ColumnDistinctCountChange7DaysCheckSpec"
    ] = UNSET
    profile_distinct_count_change_30_days: Union[
        Unset, "ColumnDistinctCountChange30DaysCheckSpec"
    ] = UNSET
    profile_distinct_percent_change_1_day: Union[
        Unset, "ColumnDistinctPercentChange1DayCheckSpec"
    ] = UNSET
    profile_distinct_percent_change_7_days: Union[
        Unset, "ColumnDistinctPercentChange7DaysCheckSpec"
    ] = UNSET
    profile_distinct_percent_change_30_days: Union[
        Unset, "ColumnDistinctPercentChange30DaysCheckSpec"
    ] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        custom_checks: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.custom_checks, Unset):
            custom_checks = self.custom_checks.to_dict()

        profile_distinct_count: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.profile_distinct_count, Unset):
            profile_distinct_count = self.profile_distinct_count.to_dict()

        profile_distinct_percent: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.profile_distinct_percent, Unset):
            profile_distinct_percent = self.profile_distinct_percent.to_dict()

        profile_duplicate_count: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.profile_duplicate_count, Unset):
            profile_duplicate_count = self.profile_duplicate_count.to_dict()

        profile_duplicate_percent: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.profile_duplicate_percent, Unset):
            profile_duplicate_percent = self.profile_duplicate_percent.to_dict()

        profile_distinct_count_anomaly: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.profile_distinct_count_anomaly, Unset):
            profile_distinct_count_anomaly = (
                self.profile_distinct_count_anomaly.to_dict()
            )

        profile_distinct_percent_anomaly: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.profile_distinct_percent_anomaly, Unset):
            profile_distinct_percent_anomaly = (
                self.profile_distinct_percent_anomaly.to_dict()
            )

        profile_distinct_count_change: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.profile_distinct_count_change, Unset):
            profile_distinct_count_change = self.profile_distinct_count_change.to_dict()

        profile_distinct_percent_change: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.profile_distinct_percent_change, Unset):
            profile_distinct_percent_change = (
                self.profile_distinct_percent_change.to_dict()
            )

        profile_distinct_count_change_1_day: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.profile_distinct_count_change_1_day, Unset):
            profile_distinct_count_change_1_day = (
                self.profile_distinct_count_change_1_day.to_dict()
            )

        profile_distinct_count_change_7_days: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.profile_distinct_count_change_7_days, Unset):
            profile_distinct_count_change_7_days = (
                self.profile_distinct_count_change_7_days.to_dict()
            )

        profile_distinct_count_change_30_days: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.profile_distinct_count_change_30_days, Unset):
            profile_distinct_count_change_30_days = (
                self.profile_distinct_count_change_30_days.to_dict()
            )

        profile_distinct_percent_change_1_day: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.profile_distinct_percent_change_1_day, Unset):
            profile_distinct_percent_change_1_day = (
                self.profile_distinct_percent_change_1_day.to_dict()
            )

        profile_distinct_percent_change_7_days: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.profile_distinct_percent_change_7_days, Unset):
            profile_distinct_percent_change_7_days = (
                self.profile_distinct_percent_change_7_days.to_dict()
            )

        profile_distinct_percent_change_30_days: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.profile_distinct_percent_change_30_days, Unset):
            profile_distinct_percent_change_30_days = (
                self.profile_distinct_percent_change_30_days.to_dict()
            )

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if custom_checks is not UNSET:
            field_dict["custom_checks"] = custom_checks
        if profile_distinct_count is not UNSET:
            field_dict["profile_distinct_count"] = profile_distinct_count
        if profile_distinct_percent is not UNSET:
            field_dict["profile_distinct_percent"] = profile_distinct_percent
        if profile_duplicate_count is not UNSET:
            field_dict["profile_duplicate_count"] = profile_duplicate_count
        if profile_duplicate_percent is not UNSET:
            field_dict["profile_duplicate_percent"] = profile_duplicate_percent
        if profile_distinct_count_anomaly is not UNSET:
            field_dict["profile_distinct_count_anomaly"] = (
                profile_distinct_count_anomaly
            )
        if profile_distinct_percent_anomaly is not UNSET:
            field_dict["profile_distinct_percent_anomaly"] = (
                profile_distinct_percent_anomaly
            )
        if profile_distinct_count_change is not UNSET:
            field_dict["profile_distinct_count_change"] = profile_distinct_count_change
        if profile_distinct_percent_change is not UNSET:
            field_dict["profile_distinct_percent_change"] = (
                profile_distinct_percent_change
            )
        if profile_distinct_count_change_1_day is not UNSET:
            field_dict["profile_distinct_count_change_1_day"] = (
                profile_distinct_count_change_1_day
            )
        if profile_distinct_count_change_7_days is not UNSET:
            field_dict["profile_distinct_count_change_7_days"] = (
                profile_distinct_count_change_7_days
            )
        if profile_distinct_count_change_30_days is not UNSET:
            field_dict["profile_distinct_count_change_30_days"] = (
                profile_distinct_count_change_30_days
            )
        if profile_distinct_percent_change_1_day is not UNSET:
            field_dict["profile_distinct_percent_change_1_day"] = (
                profile_distinct_percent_change_1_day
            )
        if profile_distinct_percent_change_7_days is not UNSET:
            field_dict["profile_distinct_percent_change_7_days"] = (
                profile_distinct_percent_change_7_days
            )
        if profile_distinct_percent_change_30_days is not UNSET:
            field_dict["profile_distinct_percent_change_30_days"] = (
                profile_distinct_percent_change_30_days
            )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.column_distinct_count_anomaly_differencing_check_spec import (
            ColumnDistinctCountAnomalyDifferencingCheckSpec,
        )
        from ..models.column_distinct_count_change_1_day_check_spec import (
            ColumnDistinctCountChange1DayCheckSpec,
        )
        from ..models.column_distinct_count_change_7_days_check_spec import (
            ColumnDistinctCountChange7DaysCheckSpec,
        )
        from ..models.column_distinct_count_change_30_days_check_spec import (
            ColumnDistinctCountChange30DaysCheckSpec,
        )
        from ..models.column_distinct_count_change_check_spec import (
            ColumnDistinctCountChangeCheckSpec,
        )
        from ..models.column_distinct_count_check_spec import (
            ColumnDistinctCountCheckSpec,
        )
        from ..models.column_distinct_percent_anomaly_stationary_check_spec import (
            ColumnDistinctPercentAnomalyStationaryCheckSpec,
        )
        from ..models.column_distinct_percent_change_1_day_check_spec import (
            ColumnDistinctPercentChange1DayCheckSpec,
        )
        from ..models.column_distinct_percent_change_7_days_check_spec import (
            ColumnDistinctPercentChange7DaysCheckSpec,
        )
        from ..models.column_distinct_percent_change_30_days_check_spec import (
            ColumnDistinctPercentChange30DaysCheckSpec,
        )
        from ..models.column_distinct_percent_change_check_spec import (
            ColumnDistinctPercentChangeCheckSpec,
        )
        from ..models.column_distinct_percent_check_spec import (
            ColumnDistinctPercentCheckSpec,
        )
        from ..models.column_duplicate_count_check_spec import (
            ColumnDuplicateCountCheckSpec,
        )
        from ..models.column_duplicate_percent_check_spec import (
            ColumnDuplicatePercentCheckSpec,
        )
        from ..models.column_uniqueness_profiling_checks_spec_custom_checks import (
            ColumnUniquenessProfilingChecksSpecCustomChecks,
        )

        d = src_dict.copy()
        _custom_checks = d.pop("custom_checks", UNSET)
        custom_checks: Union[Unset, ColumnUniquenessProfilingChecksSpecCustomChecks]
        if isinstance(_custom_checks, Unset):
            custom_checks = UNSET
        else:
            custom_checks = ColumnUniquenessProfilingChecksSpecCustomChecks.from_dict(
                _custom_checks
            )

        _profile_distinct_count = d.pop("profile_distinct_count", UNSET)
        profile_distinct_count: Union[Unset, ColumnDistinctCountCheckSpec]
        if isinstance(_profile_distinct_count, Unset):
            profile_distinct_count = UNSET
        else:
            profile_distinct_count = ColumnDistinctCountCheckSpec.from_dict(
                _profile_distinct_count
            )

        _profile_distinct_percent = d.pop("profile_distinct_percent", UNSET)
        profile_distinct_percent: Union[Unset, ColumnDistinctPercentCheckSpec]
        if isinstance(_profile_distinct_percent, Unset):
            profile_distinct_percent = UNSET
        else:
            profile_distinct_percent = ColumnDistinctPercentCheckSpec.from_dict(
                _profile_distinct_percent
            )

        _profile_duplicate_count = d.pop("profile_duplicate_count", UNSET)
        profile_duplicate_count: Union[Unset, ColumnDuplicateCountCheckSpec]
        if isinstance(_profile_duplicate_count, Unset):
            profile_duplicate_count = UNSET
        else:
            profile_duplicate_count = ColumnDuplicateCountCheckSpec.from_dict(
                _profile_duplicate_count
            )

        _profile_duplicate_percent = d.pop("profile_duplicate_percent", UNSET)
        profile_duplicate_percent: Union[Unset, ColumnDuplicatePercentCheckSpec]
        if isinstance(_profile_duplicate_percent, Unset):
            profile_duplicate_percent = UNSET
        else:
            profile_duplicate_percent = ColumnDuplicatePercentCheckSpec.from_dict(
                _profile_duplicate_percent
            )

        _profile_distinct_count_anomaly = d.pop("profile_distinct_count_anomaly", UNSET)
        profile_distinct_count_anomaly: Union[
            Unset, ColumnDistinctCountAnomalyDifferencingCheckSpec
        ]
        if isinstance(_profile_distinct_count_anomaly, Unset):
            profile_distinct_count_anomaly = UNSET
        else:
            profile_distinct_count_anomaly = (
                ColumnDistinctCountAnomalyDifferencingCheckSpec.from_dict(
                    _profile_distinct_count_anomaly
                )
            )

        _profile_distinct_percent_anomaly = d.pop(
            "profile_distinct_percent_anomaly", UNSET
        )
        profile_distinct_percent_anomaly: Union[
            Unset, ColumnDistinctPercentAnomalyStationaryCheckSpec
        ]
        if isinstance(_profile_distinct_percent_anomaly, Unset):
            profile_distinct_percent_anomaly = UNSET
        else:
            profile_distinct_percent_anomaly = (
                ColumnDistinctPercentAnomalyStationaryCheckSpec.from_dict(
                    _profile_distinct_percent_anomaly
                )
            )

        _profile_distinct_count_change = d.pop("profile_distinct_count_change", UNSET)
        profile_distinct_count_change: Union[Unset, ColumnDistinctCountChangeCheckSpec]
        if isinstance(_profile_distinct_count_change, Unset):
            profile_distinct_count_change = UNSET
        else:
            profile_distinct_count_change = (
                ColumnDistinctCountChangeCheckSpec.from_dict(
                    _profile_distinct_count_change
                )
            )

        _profile_distinct_percent_change = d.pop(
            "profile_distinct_percent_change", UNSET
        )
        profile_distinct_percent_change: Union[
            Unset, ColumnDistinctPercentChangeCheckSpec
        ]
        if isinstance(_profile_distinct_percent_change, Unset):
            profile_distinct_percent_change = UNSET
        else:
            profile_distinct_percent_change = (
                ColumnDistinctPercentChangeCheckSpec.from_dict(
                    _profile_distinct_percent_change
                )
            )

        _profile_distinct_count_change_1_day = d.pop(
            "profile_distinct_count_change_1_day", UNSET
        )
        profile_distinct_count_change_1_day: Union[
            Unset, ColumnDistinctCountChange1DayCheckSpec
        ]
        if isinstance(_profile_distinct_count_change_1_day, Unset):
            profile_distinct_count_change_1_day = UNSET
        else:
            profile_distinct_count_change_1_day = (
                ColumnDistinctCountChange1DayCheckSpec.from_dict(
                    _profile_distinct_count_change_1_day
                )
            )

        _profile_distinct_count_change_7_days = d.pop(
            "profile_distinct_count_change_7_days", UNSET
        )
        profile_distinct_count_change_7_days: Union[
            Unset, ColumnDistinctCountChange7DaysCheckSpec
        ]
        if isinstance(_profile_distinct_count_change_7_days, Unset):
            profile_distinct_count_change_7_days = UNSET
        else:
            profile_distinct_count_change_7_days = (
                ColumnDistinctCountChange7DaysCheckSpec.from_dict(
                    _profile_distinct_count_change_7_days
                )
            )

        _profile_distinct_count_change_30_days = d.pop(
            "profile_distinct_count_change_30_days", UNSET
        )
        profile_distinct_count_change_30_days: Union[
            Unset, ColumnDistinctCountChange30DaysCheckSpec
        ]
        if isinstance(_profile_distinct_count_change_30_days, Unset):
            profile_distinct_count_change_30_days = UNSET
        else:
            profile_distinct_count_change_30_days = (
                ColumnDistinctCountChange30DaysCheckSpec.from_dict(
                    _profile_distinct_count_change_30_days
                )
            )

        _profile_distinct_percent_change_1_day = d.pop(
            "profile_distinct_percent_change_1_day", UNSET
        )
        profile_distinct_percent_change_1_day: Union[
            Unset, ColumnDistinctPercentChange1DayCheckSpec
        ]
        if isinstance(_profile_distinct_percent_change_1_day, Unset):
            profile_distinct_percent_change_1_day = UNSET
        else:
            profile_distinct_percent_change_1_day = (
                ColumnDistinctPercentChange1DayCheckSpec.from_dict(
                    _profile_distinct_percent_change_1_day
                )
            )

        _profile_distinct_percent_change_7_days = d.pop(
            "profile_distinct_percent_change_7_days", UNSET
        )
        profile_distinct_percent_change_7_days: Union[
            Unset, ColumnDistinctPercentChange7DaysCheckSpec
        ]
        if isinstance(_profile_distinct_percent_change_7_days, Unset):
            profile_distinct_percent_change_7_days = UNSET
        else:
            profile_distinct_percent_change_7_days = (
                ColumnDistinctPercentChange7DaysCheckSpec.from_dict(
                    _profile_distinct_percent_change_7_days
                )
            )

        _profile_distinct_percent_change_30_days = d.pop(
            "profile_distinct_percent_change_30_days", UNSET
        )
        profile_distinct_percent_change_30_days: Union[
            Unset, ColumnDistinctPercentChange30DaysCheckSpec
        ]
        if isinstance(_profile_distinct_percent_change_30_days, Unset):
            profile_distinct_percent_change_30_days = UNSET
        else:
            profile_distinct_percent_change_30_days = (
                ColumnDistinctPercentChange30DaysCheckSpec.from_dict(
                    _profile_distinct_percent_change_30_days
                )
            )

        column_uniqueness_profiling_checks_spec = cls(
            custom_checks=custom_checks,
            profile_distinct_count=profile_distinct_count,
            profile_distinct_percent=profile_distinct_percent,
            profile_duplicate_count=profile_duplicate_count,
            profile_duplicate_percent=profile_duplicate_percent,
            profile_distinct_count_anomaly=profile_distinct_count_anomaly,
            profile_distinct_percent_anomaly=profile_distinct_percent_anomaly,
            profile_distinct_count_change=profile_distinct_count_change,
            profile_distinct_percent_change=profile_distinct_percent_change,
            profile_distinct_count_change_1_day=profile_distinct_count_change_1_day,
            profile_distinct_count_change_7_days=profile_distinct_count_change_7_days,
            profile_distinct_count_change_30_days=profile_distinct_count_change_30_days,
            profile_distinct_percent_change_1_day=profile_distinct_percent_change_1_day,
            profile_distinct_percent_change_7_days=profile_distinct_percent_change_7_days,
            profile_distinct_percent_change_30_days=profile_distinct_percent_change_30_days,
        )

        column_uniqueness_profiling_checks_spec.additional_properties = d
        return column_uniqueness_profiling_checks_spec

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.sensor_folder_model_folders import SensorFolderModelFolders
    from ..models.sensor_list_model import SensorListModel


T = TypeVar("T", bound="SensorFolderModel")


@_attrs_define
class SensorFolderModel:
    """Sensor folder model that contains sensors defined in this folder or a list of nested folders with sensors.

    Attributes:
        folders (Union[Unset, SensorFolderModelFolders]): A dictionary of nested folders with sensors, the keys are the
            folder names.
        sensors (Union[Unset, List['SensorListModel']]): List of sensors defined in this folder.
        all_sensors (Union[Unset, List['SensorListModel']]):
    """

    folders: Union[Unset, "SensorFolderModelFolders"] = UNSET
    sensors: Union[Unset, List["SensorListModel"]] = UNSET
    all_sensors: Union[Unset, List["SensorListModel"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        folders: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.folders, Unset):
            folders = self.folders.to_dict()

        sensors: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.sensors, Unset):
            sensors = []
            for sensors_item_data in self.sensors:
                sensors_item = sensors_item_data.to_dict()

                sensors.append(sensors_item)

        all_sensors: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.all_sensors, Unset):
            all_sensors = []
            for all_sensors_item_data in self.all_sensors:
                all_sensors_item = all_sensors_item_data.to_dict()

                all_sensors.append(all_sensors_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if folders is not UNSET:
            field_dict["folders"] = folders
        if sensors is not UNSET:
            field_dict["sensors"] = sensors
        if all_sensors is not UNSET:
            field_dict["all_sensors"] = all_sensors

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.sensor_folder_model_folders import SensorFolderModelFolders
        from ..models.sensor_list_model import SensorListModel

        d = src_dict.copy()
        _folders = d.pop("folders", UNSET)
        folders: Union[Unset, SensorFolderModelFolders]
        if isinstance(_folders, Unset):
            folders = UNSET
        else:
            folders = SensorFolderModelFolders.from_dict(_folders)

        sensors = []
        _sensors = d.pop("sensors", UNSET)
        for sensors_item_data in _sensors or []:
            sensors_item = SensorListModel.from_dict(sensors_item_data)

            sensors.append(sensors_item)

        all_sensors = []
        _all_sensors = d.pop("all_sensors", UNSET)
        for all_sensors_item_data in _all_sensors or []:
            all_sensors_item = SensorListModel.from_dict(all_sensors_item_data)

            all_sensors.append(all_sensors_item)

        sensor_folder_model = cls(
            folders=folders,
            sensors=sensors,
            all_sensors=all_sensors,
        )

        sensor_folder_model.additional_properties = d
        return sensor_folder_model

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="IncidentNotificationTargetSpec")


@_attrs_define
class IncidentNotificationTargetSpec:
    """
    Attributes:
        incident_opened_addresses (Union[Unset, str]): Notification address(es) where the notification messages
            describing new incidents are pushed using a HTTP POST request (for webhook address) or an SMTP (for email
            address). The format of the JSON message is documented in the IncidentNotificationMessage object.
        incident_acknowledged_addresses (Union[Unset, str]): Notification address(es) where the notification messages
            describing acknowledged messages are pushed using a HTTP POST request (for webhook address) or an SMTP (for
            email address). The format of the JSON message is documented in the IncidentNotificationMessage object.
        incident_resolved_addresses (Union[Unset, str]): Notification address(es) where the notification messages
            describing resolved messages are pushed using a HTTP POST request (for webhook address) or an SMTP (for email
            address). The format of the JSON message is documented in the IncidentNotificationMessage object.
        incident_muted_addresses (Union[Unset, str]): Notification address(es) where the notification messages
            describing muted messages are pushed using a HTTP POST request (for webhook address) or an SMTP (for email
            address). The format of the JSON message is documented in the IncidentNotificationMessage object.
    """

    incident_opened_addresses: Union[Unset, str] = UNSET
    incident_acknowledged_addresses: Union[Unset, str] = UNSET
    incident_resolved_addresses: Union[Unset, str] = UNSET
    incident_muted_addresses: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        incident_opened_addresses = self.incident_opened_addresses
        incident_acknowledged_addresses = self.incident_acknowledged_addresses
        incident_resolved_addresses = self.incident_resolved_addresses
        incident_muted_addresses = self.incident_muted_addresses

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if incident_opened_addresses is not UNSET:
            field_dict["incident_opened_addresses"] = incident_opened_addresses
        if incident_acknowledged_addresses is not UNSET:
            field_dict["incident_acknowledged_addresses"] = (
                incident_acknowledged_addresses
            )
        if incident_resolved_addresses is not UNSET:
            field_dict["incident_resolved_addresses"] = incident_resolved_addresses
        if incident_muted_addresses is not UNSET:
            field_dict["incident_muted_addresses"] = incident_muted_addresses

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        incident_opened_addresses = d.pop("incident_opened_addresses", UNSET)

        incident_acknowledged_addresses = d.pop(
            "incident_acknowledged_addresses", UNSET
        )

        incident_resolved_addresses = d.pop("incident_resolved_addresses", UNSET)

        incident_muted_addresses = d.pop("incident_muted_addresses", UNSET)

        incident_notification_target_spec = cls(
            incident_opened_addresses=incident_opened_addresses,
            incident_acknowledged_addresses=incident_acknowledged_addresses,
            incident_resolved_addresses=incident_resolved_addresses,
            incident_muted_addresses=incident_muted_addresses,
        )

        incident_notification_target_spec.additional_properties = d
        return incident_notification_target_spec

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="IncidentsPerConnectionModel")


@_attrs_define
class IncidentsPerConnectionModel:
    """
    Attributes:
        connection (Union[Unset, str]): Connection (data source) name.
        open_incidents (Union[Unset, int]): Count of open (new) data quality incidents.
        most_recent_first_seen (Union[Unset, int]): The UTC timestamp when the most recent data quality incident was
            first seen.
    """

    connection: Union[Unset, str] = UNSET
    open_incidents: Union[Unset, int] = UNSET
    most_recent_first_seen: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        connection = self.connection
        open_incidents = self.open_incidents
        most_recent_first_seen = self.most_recent_first_seen

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if connection is not UNSET:
            field_dict["connection"] = connection
        if open_incidents is not UNSET:
            field_dict["openIncidents"] = open_incidents
        if most_recent_first_seen is not UNSET:
            field_dict["mostRecentFirstSeen"] = most_recent_first_seen

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        connection = d.pop("connection", UNSET)

        open_incidents = d.pop("openIncidents", UNSET)

        most_recent_first_seen = d.pop("mostRecentFirstSeen", UNSET)

        incidents_per_connection_model = cls(
            connection=connection,
            open_incidents=open_incidents,
            most_recent_first_seen=most_recent_first_seen,
        )

        incidents_per_connection_model.additional_properties = d
        return incidents_per_connection_model

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="ColumnTypeSnapshotSpec")


@_attrs_define
class ColumnTypeSnapshotSpec:
    """
    Attributes:
        column_type (Union[Unset, str]): Column data type using the monitored database type names.
        nullable (Union[Unset, bool]): Column is nullable.
        length (Union[Unset, int]): Maximum length of text and binary columns.
        precision (Union[Unset, int]): Precision of a numeric (decimal) data type.
        scale (Union[Unset, int]): Scale of a numeric (decimal) data type.
        nested (Union[Unset, bool]): This field is a nested field inside another STRUCT. It is used to identify nested
            fields in JSON files.
    """

    column_type: Union[Unset, str] = UNSET
    nullable: Union[Unset, bool] = UNSET
    length: Union[Unset, int] = UNSET
    precision: Union[Unset, int] = UNSET
    scale: Union[Unset, int] = UNSET
    nested: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        column_type = self.column_type
        nullable = self.nullable
        length = self.length
        precision = self.precision
        scale = self.scale
        nested = self.nested

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if column_type is not UNSET:
            field_dict["column_type"] = column_type
        if nullable is not UNSET:
            field_dict["nullable"] = nullable
        if length is not UNSET:
            field_dict["length"] = length
        if precision is not UNSET:
            field_dict["precision"] = precision
        if scale is not UNSET:
            field_dict["scale"] = scale
        if nested is not UNSET:
            field_dict["nested"] = nested

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        column_type = d.pop("column_type", UNSET)

        nullable = d.pop("nullable", UNSET)

        length = d.pop("length", UNSET)

        precision = d.pop("precision", UNSET)

        scale = d.pop("scale", UNSET)

        nested = d.pop("nested", UNSET)

        column_type_snapshot_spec = cls(
            column_type=column_type,
            nullable=nullable,
            length=length,
            precision=precision,
            scale=scale,
            nested=nested,
        )

        column_type_snapshot_spec.additional_properties = d
        return column_type_snapshot_spec

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.column_quality_policy_spec import ColumnQualityPolicySpec


T = TypeVar("T", bound="ColumnQualityPolicyModel")


@_attrs_define
class ColumnQualityPolicyModel:
    """Default column-level checks pattern (data quality policy) model

    Attributes:
        policy_name (Union[Unset, str]): Quality policy name
        policy_spec (Union[Unset, ColumnQualityPolicySpec]):
        can_edit (Union[Unset, bool]): Boolean flag that decides if the current user can update or delete this object.
        yaml_parsing_error (Union[Unset, str]): Optional parsing error that was captured when parsing the YAML file.
            This field is null when the YAML file is valid. If an error was captured, this field returns the file parsing
            error message and the file location.
    """

    policy_name: Union[Unset, str] = UNSET
    policy_spec: Union[Unset, "ColumnQualityPolicySpec"] = UNSET
    can_edit: Union[Unset, bool] = UNSET
    yaml_parsing_error: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        policy_name = self.policy_name
        policy_spec: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.policy_spec, Unset):
            policy_spec = self.policy_spec.to_dict()

        can_edit = self.can_edit
        yaml_parsing_error = self.yaml_parsing_error

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if policy_name is not UNSET:
            field_dict["policy_name"] = policy_name
        if policy_spec is not UNSET:
            field_dict["policy_spec"] = policy_spec
        if can_edit is not UNSET:
            field_dict["can_edit"] = can_edit
        if yaml_parsing_error is not UNSET:
            field_dict["yaml_parsing_error"] = yaml_parsing_error

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.column_quality_policy_spec import ColumnQualityPolicySpec

        d = src_dict.copy()
        policy_name = d.pop("policy_name", UNSET)

        _policy_spec = d.pop("policy_spec", UNSET)
        policy_spec: Union[Unset, ColumnQualityPolicySpec]
        if isinstance(_policy_spec, Unset):
            policy_spec = UNSET
        else:
            policy_spec = ColumnQualityPolicySpec.from_dict(_policy_spec)

        can_edit = d.pop("can_edit", UNSET)

        yaml_parsing_error = d.pop("yaml_parsing_error", UNSET)

        column_quality_policy_model = cls(
            policy_name=policy_name,
            policy_spec=policy_spec,
            can_edit=can_edit,
            yaml_parsing_error=yaml_parsing_error,
        )

        column_quality_policy_model.additional_properties = d
        return column_quality_policy_model

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.column_uniqueness_distinct_count_statistics_collector_spec import (
        ColumnUniquenessDistinctCountStatisticsCollectorSpec,
    )
    from ..models.column_uniqueness_distinct_percent_statistics_collector_spec import (
        ColumnUniquenessDistinctPercentStatisticsCollectorSpec,
    )
    from ..models.column_uniqueness_duplicate_count_statistics_collector_spec import (
        ColumnUniquenessDuplicateCountStatisticsCollectorSpec,
    )
    from ..models.column_uniqueness_duplicate_percent_statistics_collector_spec import (
        ColumnUniquenessDuplicatePercentStatisticsCollectorSpec,
    )


T = TypeVar("T", bound="ColumnUniquenessStatisticsCollectorsSpec")


@_attrs_define
class ColumnUniquenessStatisticsCollectorsSpec:
    """
    Attributes:
        distinct_count (Union[Unset, ColumnUniquenessDistinctCountStatisticsCollectorSpec]):
        distinct_percent (Union[Unset, ColumnUniquenessDistinctPercentStatisticsCollectorSpec]):
        duplicate_count (Union[Unset, ColumnUniquenessDuplicateCountStatisticsCollectorSpec]):
        duplicate_percent (Union[Unset, ColumnUniquenessDuplicatePercentStatisticsCollectorSpec]):
    """

    distinct_count: Union[
        Unset, "ColumnUniquenessDistinctCountStatisticsCollectorSpec"
    ] = UNSET
    distinct_percent: Union[
        Unset, "ColumnUniquenessDistinctPercentStatisticsCollectorSpec"
    ] = UNSET
    duplicate_count: Union[
        Unset, "ColumnUniquenessDuplicateCountStatisticsCollectorSpec"
    ] = UNSET
    duplicate_percent: Union[
        Unset, "ColumnUniquenessDuplicatePercentStatisticsCollectorSpec"
    ] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        distinct_count: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.distinct_count, Unset):
            distinct_count = self.distinct_count.to_dict()

        distinct_percent: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.distinct_percent, Unset):
            distinct_percent = self.distinct_percent.to_dict()

        duplicate_count: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.duplicate_count, Unset):
            duplicate_count = self.duplicate_count.to_dict()

        duplicate_percent: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.duplicate_percent, Unset):
            duplicate_percent = self.duplicate_percent.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if distinct_count is not UNSET:
            field_dict["distinct_count"] = distinct_count
        if distinct_percent is not UNSET:
            field_dict["distinct_percent"] = distinct_percent
        if duplicate_count is not UNSET:
            field_dict["duplicate_count"] = duplicate_count
        if duplicate_percent is not UNSET:
            field_dict["duplicate_percent"] = duplicate_percent

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.column_uniqueness_distinct_count_statistics_collector_spec import (
            ColumnUniquenessDistinctCountStatisticsCollectorSpec,
        )
        from ..models.column_uniqueness_distinct_percent_statistics_collector_spec import (
            ColumnUniquenessDistinctPercentStatisticsCollectorSpec,
        )
        from ..models.column_uniqueness_duplicate_count_statistics_collector_spec import (
            ColumnUniquenessDuplicateCountStatisticsCollectorSpec,
        )
        from ..models.column_uniqueness_duplicate_percent_statistics_collector_spec import (
            ColumnUniquenessDuplicatePercentStatisticsCollectorSpec,
        )

        d = src_dict.copy()
        _distinct_count = d.pop("distinct_count", UNSET)
        distinct_count: Union[
            Unset, ColumnUniquenessDistinctCountStatisticsCollectorSpec
        ]
        if isinstance(_distinct_count, Unset):
            distinct_count = UNSET
        else:
            distinct_count = (
                ColumnUniquenessDistinctCountStatisticsCollectorSpec.from_dict(
                    _distinct_count
                )
            )

        _distinct_percent = d.pop("distinct_percent", UNSET)
        distinct_percent: Union[
            Unset, ColumnUniquenessDistinctPercentStatisticsCollectorSpec
        ]
        if isinstance(_distinct_percent, Unset):
            distinct_percent = UNSET
        else:
            distinct_percent = (
                ColumnUniquenessDistinctPercentStatisticsCollectorSpec.from_dict(
                    _distinct_percent
                )
            )

        _duplicate_count = d.pop("duplicate_count", UNSET)
        duplicate_count: Union[
            Unset, ColumnUniquenessDuplicateCountStatisticsCollectorSpec
        ]
        if isinstance(_duplicate_count, Unset):
            duplicate_count = UNSET
        else:
            duplicate_count = (
                ColumnUniquenessDuplicateCountStatisticsCollectorSpec.from_dict(
                    _duplicate_count
                )
            )

        _duplicate_percent = d.pop("duplicate_percent", UNSET)
        duplicate_percent: Union[
            Unset, ColumnUniquenessDuplicatePercentStatisticsCollectorSpec
        ]
        if isinstance(_duplicate_percent, Unset):
            duplicate_percent = UNSET
        else:
            duplicate_percent = (
                ColumnUniquenessDuplicatePercentStatisticsCollectorSpec.from_dict(
                    _duplicate_percent
                )
            )

        column_uniqueness_statistics_collectors_spec = cls(
            distinct_count=distinct_count,
            distinct_percent=distinct_percent,
            duplicate_count=duplicate_count,
            duplicate_percent=duplicate_percent,
        )

        column_uniqueness_statistics_collectors_spec.additional_properties = d
        return column_uniqueness_statistics_collectors_spec

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="RepairStoredDataQueueJobParameters")


@_attrs_define
class RepairStoredDataQueueJobParameters:
    """
    Attributes:
        connection_name (Union[Unset, str]):
        schema_table_name (Union[Unset, str]):
        repair_errors (Union[Unset, bool]):
        repair_statistics (Union[Unset, bool]):
        repair_check_results (Union[Unset, bool]):
        repair_sensor_readouts (Union[Unset, bool]):
    """

    connection_name: Union[Unset, str] = UNSET
    schema_table_name: Union[Unset, str] = UNSET
    repair_errors: Union[Unset, bool] = UNSET
    repair_statistics: Union[Unset, bool] = UNSET
    repair_check_results: Union[Unset, bool] = UNSET
    repair_sensor_readouts: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        connection_name = self.connection_name
        schema_table_name = self.schema_table_name
        repair_errors = self.repair_errors
        repair_statistics = self.repair_statistics
        repair_check_results = self.repair_check_results
        repair_sensor_readouts = self.repair_sensor_readouts

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if connection_name is not UNSET:
            field_dict["connectionName"] = connection_name
        if schema_table_name is not UNSET:
            field_dict["schemaTableName"] = schema_table_name
        if repair_errors is not UNSET:
            field_dict["repairErrors"] = repair_errors
        if repair_statistics is not UNSET:
            field_dict["repairStatistics"] = repair_statistics
        if repair_check_results is not UNSET:
            field_dict["repairCheckResults"] = repair_check_results
        if repair_sensor_readouts is not UNSET:
            field_dict["repairSensorReadouts"] = repair_sensor_readouts

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        connection_name = d.pop("connectionName", UNSET)

        schema_table_name = d.pop("schemaTableName", UNSET)

        repair_errors = d.pop("repairErrors", UNSET)

        repair_statistics = d.pop("repairStatistics", UNSET)

        repair_check_results = d.pop("repairCheckResults", UNSET)

        repair_sensor_readouts = d.pop("repairSensorReadouts", UNSET)

        repair_stored_data_queue_job_parameters = cls(
            connection_name=connection_name,
            schema_table_name=schema_table_name,
            repair_errors=repair_errors,
            repair_statistics=repair_statistics,
            repair_check_results=repair_check_results,
            repair_sensor_readouts=repair_sensor_readouts,
        )

        repair_stored_data_queue_job_parameters.additional_properties = d
        return repair_stored_data_queue_job_parameters

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.incident_model import IncidentModel


T = TypeVar("T", bound="TopIncidentsModelTopIncidents")


@_attrs_define
class TopIncidentsModelTopIncidents:
    """Dictionary of the top incidents, grouped by the grouping such as the data quality dimension or a data quality check
    category. The incidents are sorted by the first seen descending (the most recent first).

    """

    additional_properties: Dict[str, List["IncidentModel"]] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        pass

        field_dict: Dict[str, Any] = {}
        for prop_name, prop in self.additional_properties.items():
            field_dict[prop_name] = []
            for additional_property_item_data in prop:
                additional_property_item = additional_property_item_data.to_dict()

                field_dict[prop_name].append(additional_property_item)

        field_dict.update({})

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.incident_model import IncidentModel

        d = src_dict.copy()
        top_incidents_model_top_incidents = cls()

        additional_properties = {}
        for prop_name, prop_dict in d.items():
            additional_property = []
            _additional_property = prop_dict
            for additional_property_item_data in _additional_property:
                additional_property_item = IncidentModel.from_dict(
                    additional_property_item_data
                )

                additional_property.append(additional_property_item)

            additional_properties[prop_name] = additional_property

        top_incidents_model_top_incidents.additional_properties = additional_properties
        return top_incidents_model_top_incidents

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> List["IncidentModel"]:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: List["IncidentModel"]) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.statistics_data_scope import StatisticsDataScope
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.collect_statistics_result import CollectStatisticsResult
    from ..models.statistics_collector_search_filters import (
        StatisticsCollectorSearchFilters,
    )


T = TypeVar("T", bound="CollectStatisticsQueueJobParameters")


@_attrs_define
class CollectStatisticsQueueJobParameters:
    """
    Attributes:
        statistics_collector_search_filters (Union[Unset, StatisticsCollectorSearchFilters]):
        data_scope (Union[Unset, StatisticsDataScope]):
        configure_table (Union[Unset, bool]): Turns on a special mode of collecting statistics that will configure the
            timestamp and ID columns. It should be used only during the first statistics collection.
        samples_limit (Union[Unset, int]): The default limit of column samples that are collected.
        dummy_sensor_execution (Union[Unset, bool]): Boolean flag that enables a dummy statistics collection (sensors
            are executed, but the statistics results are not written to the parquet files).
        collect_statistics_result (Union[Unset, CollectStatisticsResult]): Returns the result with the summary of the
            statistics collected.
    """

    statistics_collector_search_filters: Union[
        Unset, "StatisticsCollectorSearchFilters"
    ] = UNSET
    data_scope: Union[Unset, StatisticsDataScope] = UNSET
    configure_table: Union[Unset, bool] = UNSET
    samples_limit: Union[Unset, int] = UNSET
    dummy_sensor_execution: Union[Unset, bool] = UNSET
    collect_statistics_result: Union[Unset, "CollectStatisticsResult"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        statistics_collector_search_filters: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.statistics_collector_search_filters, Unset):
            statistics_collector_search_filters = (
                self.statistics_collector_search_filters.to_dict()
            )

        data_scope: Union[Unset, str] = UNSET
        if not isinstance(self.data_scope, Unset):
            data_scope = self.data_scope.value

        configure_table = self.configure_table
        samples_limit = self.samples_limit
        dummy_sensor_execution = self.dummy_sensor_execution
        collect_statistics_result: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.collect_statistics_result, Unset):
            collect_statistics_result = self.collect_statistics_result.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if statistics_collector_search_filters is not UNSET:
            field_dict["statistics_collector_search_filters"] = (
                statistics_collector_search_filters
            )
        if data_scope is not UNSET:
            field_dict["data_scope"] = data_scope
        if configure_table is not UNSET:
            field_dict["configure_table"] = configure_table
        if samples_limit is not UNSET:
            field_dict["samples_limit"] = samples_limit
        if dummy_sensor_execution is not UNSET:
            field_dict["dummy_sensor_execution"] = dummy_sensor_execution
        if collect_statistics_result is not UNSET:
            field_dict["collect_statistics_result"] = collect_statistics_result

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.collect_statistics_result import CollectStatisticsResult
        from ..models.statistics_collector_search_filters import (
            StatisticsCollectorSearchFilters,
        )

        d = src_dict.copy()
        _statistics_collector_search_filters = d.pop(
            "statistics_collector_search_filters", UNSET
        )
        statistics_collector_search_filters: Union[
            Unset, StatisticsCollectorSearchFilters
        ]
        if isinstance(_statistics_collector_search_filters, Unset):
            statistics_collector_search_filters = UNSET
        else:
            statistics_collector_search_filters = (
                StatisticsCollectorSearchFilters.from_dict(
                    _statistics_collector_search_filters
                )
            )

        _data_scope = d.pop("data_scope", UNSET)
        data_scope: Union[Unset, StatisticsDataScope]
        if isinstance(_data_scope, Unset):
            data_scope = UNSET
        else:
            data_scope = StatisticsDataScope(_data_scope)

        configure_table = d.pop("configure_table", UNSET)

        samples_limit = d.pop("samples_limit", UNSET)

        dummy_sensor_execution = d.pop("dummy_sensor_execution", UNSET)

        _collect_statistics_result = d.pop("collect_statistics_result", UNSET)
        collect_statistics_result: Union[Unset, CollectStatisticsResult]
        if isinstance(_collect_statistics_result, Unset):
            collect_statistics_result = UNSET
        else:
            collect_statistics_result = CollectStatisticsResult.from_dict(
                _collect_statistics_result
            )

        collect_statistics_queue_job_parameters = cls(
            statistics_collector_search_filters=statistics_collector_search_filters,
            data_scope=data_scope,
            configure_table=configure_table,
            samples_limit=samples_limit,
            dummy_sensor_execution=dummy_sensor_execution,
            collect_statistics_result=collect_statistics_result,
        )

        collect_statistics_queue_job_parameters.additional_properties = d
        return collect_statistics_queue_job_parameters

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.column_sampling_column_samples_statistics_collector_spec import (
        ColumnSamplingColumnSamplesStatisticsCollectorSpec,
    )


T = TypeVar("T", bound="ColumnSamplingStatisticsCollectorsSpec")


@_attrs_define
class ColumnSamplingStatisticsCollectorsSpec:
    """
    Attributes:
        column_samples (Union[Unset, ColumnSamplingColumnSamplesStatisticsCollectorSpec]):
    """

    column_samples: Union[
        Unset, "ColumnSamplingColumnSamplesStatisticsCollectorSpec"
    ] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        column_samples: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.column_samples, Unset):
            column_samples = self.column_samples.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if column_samples is not UNSET:
            field_dict["column_samples"] = column_samples

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.column_sampling_column_samples_statistics_collector_spec import (
            ColumnSamplingColumnSamplesStatisticsCollectorSpec,
        )

        d = src_dict.copy()
        _column_samples = d.pop("column_samples", UNSET)
        column_samples: Union[Unset, ColumnSamplingColumnSamplesStatisticsCollectorSpec]
        if isinstance(_column_samples, Unset):
            column_samples = UNSET
        else:
            column_samples = (
                ColumnSamplingColumnSamplesStatisticsCollectorSpec.from_dict(
                    _column_samples
                )
            )

        column_sampling_statistics_collectors_spec = cls(
            column_samples=column_samples,
        )

        column_sampling_statistics_collectors_spec.additional_properties = d
        return column_sampling_statistics_collectors_spec

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.table_comparison_column_results_model_column_comparison_results import (
        TableComparisonColumnResultsModelColumnComparisonResults,
    )


T = TypeVar("T", bound="TableComparisonColumnResultsModel")


@_attrs_define
class TableComparisonColumnResultsModel:
    """The table comparison column results model with the information about the most recent table comparison relating to a
    single compared column.

        Attributes:
            column_name (Union[Unset, str]): Column name
            column_comparison_results (Union[Unset, TableComparisonColumnResultsModelColumnComparisonResults]): The
                dictionary of comparison results between the tables for the specific column. The keys for the dictionary are
                check names. The values are summaries of the most recent comparison on this column.
    """

    column_name: Union[Unset, str] = UNSET
    column_comparison_results: Union[
        Unset, "TableComparisonColumnResultsModelColumnComparisonResults"
    ] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        column_name = self.column_name
        column_comparison_results: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.column_comparison_results, Unset):
            column_comparison_results = self.column_comparison_results.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if column_name is not UNSET:
            field_dict["column_name"] = column_name
        if column_comparison_results is not UNSET:
            field_dict["column_comparison_results"] = column_comparison_results

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.table_comparison_column_results_model_column_comparison_results import (
            TableComparisonColumnResultsModelColumnComparisonResults,
        )

        d = src_dict.copy()
        column_name = d.pop("column_name", UNSET)

        _column_comparison_results = d.pop("column_comparison_results", UNSET)
        column_comparison_results: Union[
            Unset, TableComparisonColumnResultsModelColumnComparisonResults
        ]
        if isinstance(_column_comparison_results, Unset):
            column_comparison_results = UNSET
        else:
            column_comparison_results = (
                TableComparisonColumnResultsModelColumnComparisonResults.from_dict(
                    _column_comparison_results
                )
            )

        table_comparison_column_results_model = cls(
            column_name=column_name,
            column_comparison_results=column_comparison_results,
        )

        table_comparison_column_results_model.additional_properties = d
        return table_comparison_column_results_model

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.check_search_filters import CheckSearchFilters
    from ..models.delete_stored_data_queue_job_parameters import (
        DeleteStoredDataQueueJobParameters,
    )
    from ..models.import_tables_queue_job_parameters import (
        ImportTablesQueueJobParameters,
    )
    from ..models.statistics_collector_search_filters import (
        StatisticsCollectorSearchFilters,
    )


T = TypeVar("T", bound="SchemaModel")


@_attrs_define
class SchemaModel:
    """Schema model

    Attributes:
        connection_name (Union[Unset, str]): Connection name.
        schema_name (Union[Unset, str]): Schema name.
        directory_prefix (Union[Unset, str]): Directory prefix.
        run_checks_job_template (Union[Unset, CheckSearchFilters]): Target data quality checks filter, identifies which
            checks on which tables and columns should be executed.
        run_profiling_checks_job_template (Union[Unset, CheckSearchFilters]): Target data quality checks filter,
            identifies which checks on which tables and columns should be executed.
        run_monitoring_checks_job_template (Union[Unset, CheckSearchFilters]): Target data quality checks filter,
            identifies which checks on which tables and columns should be executed.
        run_partition_checks_job_template (Union[Unset, CheckSearchFilters]): Target data quality checks filter,
            identifies which checks on which tables and columns should be executed.
        collect_statistics_job_template (Union[Unset, StatisticsCollectorSearchFilters]):
        import_table_job_parameters (Union[Unset, ImportTablesQueueJobParameters]):
        data_clean_job_template (Union[Unset, DeleteStoredDataQueueJobParameters]):
        can_edit (Union[Unset, bool]): Boolean flag that decides if the current user can update or delete the schema.
        can_collect_statistics (Union[Unset, bool]): Boolean flag that decides if the current user can collect
            statistics.
        can_run_checks (Union[Unset, bool]): Boolean flag that decides if the current user can run checks.
        can_delete_data (Union[Unset, bool]): Boolean flag that decides if the current user can delete data (results).
        error_message (Union[Unset, str]): Field for error message.
    """

    connection_name: Union[Unset, str] = UNSET
    schema_name: Union[Unset, str] = UNSET
    directory_prefix: Union[Unset, str] = UNSET
    run_checks_job_template: Union[Unset, "CheckSearchFilters"] = UNSET
    run_profiling_checks_job_template: Union[Unset, "CheckSearchFilters"] = UNSET
    run_monitoring_checks_job_template: Union[Unset, "CheckSearchFilters"] = UNSET
    run_partition_checks_job_template: Union[Unset, "CheckSearchFilters"] = UNSET
    collect_statistics_job_template: Union[
        Unset, "StatisticsCollectorSearchFilters"
    ] = UNSET
    import_table_job_parameters: Union[Unset, "ImportTablesQueueJobParameters"] = UNSET
    data_clean_job_template: Union[Unset, "DeleteStoredDataQueueJobParameters"] = UNSET
    can_edit: Union[Unset, bool] = UNSET
    can_collect_statistics: Union[Unset, bool] = UNSET
    can_run_checks: Union[Unset, bool] = UNSET
    can_delete_data: Union[Unset, bool] = UNSET
    error_message: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        connection_name = self.connection_name
        schema_name = self.schema_name
        directory_prefix = self.directory_prefix
        run_checks_job_template: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.run_checks_job_template, Unset):
            run_checks_job_template = self.run_checks_job_template.to_dict()

        run_profiling_checks_job_template: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.run_profiling_checks_job_template, Unset):
            run_profiling_checks_job_template = (
                self.run_profiling_checks_job_template.to_dict()
            )

        run_monitoring_checks_job_template: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.run_monitoring_checks_job_template, Unset):
            run_monitoring_checks_job_template = (
                self.run_monitoring_checks_job_template.to_dict()
            )

        run_partition_checks_job_template: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.run_partition_checks_job_template, Unset):
            run_partition_checks_job_template = (
                self.run_partition_checks_job_template.to_dict()
            )

        collect_statistics_job_template: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.collect_statistics_job_template, Unset):
            collect_statistics_job_template = (
                self.collect_statistics_job_template.to_dict()
            )

        import_table_job_parameters: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.import_table_job_parameters, Unset):
            import_table_job_parameters = self.import_table_job_parameters.to_dict()

        data_clean_job_template: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.data_clean_job_template, Unset):
            data_clean_job_template = self.data_clean_job_template.to_dict()

        can_edit = self.can_edit
        can_collect_statistics = self.can_collect_statistics
        can_run_checks = self.can_run_checks
        can_delete_data = self.can_delete_data
        error_message = self.error_message

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if connection_name is not UNSET:
            field_dict["connection_name"] = connection_name
        if schema_name is not UNSET:
            field_dict["schema_name"] = schema_name
        if directory_prefix is not UNSET:
            field_dict["directory_prefix"] = directory_prefix
        if run_checks_job_template is not UNSET:
            field_dict["run_checks_job_template"] = run_checks_job_template
        if run_profiling_checks_job_template is not UNSET:
            field_dict["run_profiling_checks_job_template"] = (
                run_profiling_checks_job_template
            )
        if run_monitoring_checks_job_template is not UNSET:
            field_dict["run_monitoring_checks_job_template"] = (
                run_monitoring_checks_job_template
            )
        if run_partition_checks_job_template is not UNSET:
            field_dict["run_partition_checks_job_template"] = (
                run_partition_checks_job_template
            )
        if collect_statistics_job_template is not UNSET:
            field_dict["collect_statistics_job_template"] = (
                collect_statistics_job_template
            )
        if import_table_job_parameters is not UNSET:
            field_dict["import_table_job_parameters"] = import_table_job_parameters
        if data_clean_job_template is not UNSET:
            field_dict["data_clean_job_template"] = data_clean_job_template
        if can_edit is not UNSET:
            field_dict["can_edit"] = can_edit
        if can_collect_statistics is not UNSET:
            field_dict["can_collect_statistics"] = can_collect_statistics
        if can_run_checks is not UNSET:
            field_dict["can_run_checks"] = can_run_checks
        if can_delete_data is not UNSET:
            field_dict["can_delete_data"] = can_delete_data
        if error_message is not UNSET:
            field_dict["error_message"] = error_message

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.check_search_filters import CheckSearchFilters
        from ..models.delete_stored_data_queue_job_parameters import (
            DeleteStoredDataQueueJobParameters,
        )
        from ..models.import_tables_queue_job_parameters import (
            ImportTablesQueueJobParameters,
        )
        from ..models.statistics_collector_search_filters import (
            StatisticsCollectorSearchFilters,
        )

        d = src_dict.copy()
        connection_name = d.pop("connection_name", UNSET)

        schema_name = d.pop("schema_name", UNSET)

        directory_prefix = d.pop("directory_prefix", UNSET)

        _run_checks_job_template = d.pop("run_checks_job_template", UNSET)
        run_checks_job_template: Union[Unset, CheckSearchFilters]
        if isinstance(_run_checks_job_template, Unset):
            run_checks_job_template = UNSET
        else:
            run_checks_job_template = CheckSearchFilters.from_dict(
                _run_checks_job_template
            )

        _run_profiling_checks_job_template = d.pop(
            "run_profiling_checks_job_template", UNSET
        )
        run_profiling_checks_job_template: Union[Unset, CheckSearchFilters]
        if isinstance(_run_profiling_checks_job_template, Unset):
            run_profiling_checks_job_template = UNSET
        else:
            run_profiling_checks_job_template = CheckSearchFilters.from_dict(
                _run_profiling_checks_job_template
            )

        _run_monitoring_checks_job_template = d.pop(
            "run_monitoring_checks_job_template", UNSET
        )
        run_monitoring_checks_job_template: Union[Unset, CheckSearchFilters]
        if isinstance(_run_monitoring_checks_job_template, Unset):
            run_monitoring_checks_job_template = UNSET
        else:
            run_monitoring_checks_job_template = CheckSearchFilters.from_dict(
                _run_monitoring_checks_job_template
            )

        _run_partition_checks_job_template = d.pop(
            "run_partition_checks_job_template", UNSET
        )
        run_partition_checks_job_template: Union[Unset, CheckSearchFilters]
        if isinstance(_run_partition_checks_job_template, Unset):
            run_partition_checks_job_template = UNSET
        else:
            run_partition_checks_job_template = CheckSearchFilters.from_dict(
                _run_partition_checks_job_template
            )

        _collect_statistics_job_template = d.pop(
            "collect_statistics_job_template", UNSET
        )
        collect_statistics_job_template: Union[Unset, StatisticsCollectorSearchFilters]
        if isinstance(_collect_statistics_job_template, Unset):
            collect_statistics_job_template = UNSET
        else:
            collect_statistics_job_template = (
                StatisticsCollectorSearchFilters.from_dict(
                    _collect_statistics_job_template
                )
            )

        _import_table_job_parameters = d.pop("import_table_job_parameters", UNSET)
        import_table_job_parameters: Union[Unset, ImportTablesQueueJobParameters]
        if isinstance(_import_table_job_parameters, Unset):
            import_table_job_parameters = UNSET
        else:
            import_table_job_parameters = ImportTablesQueueJobParameters.from_dict(
                _import_table_job_parameters
            )

        _data_clean_job_template = d.pop("data_clean_job_template", UNSET)
        data_clean_job_template: Union[Unset, DeleteStoredDataQueueJobParameters]
        if isinstance(_data_clean_job_template, Unset):
            data_clean_job_template = UNSET
        else:
            data_clean_job_template = DeleteStoredDataQueueJobParameters.from_dict(
                _data_clean_job_template
            )

        can_edit = d.pop("can_edit", UNSET)

        can_collect_statistics = d.pop("can_collect_statistics", UNSET)

        can_run_checks = d.pop("can_run_checks", UNSET)

        can_delete_data = d.pop("can_delete_data", UNSET)

        error_message = d.pop("error_message", UNSET)

        schema_model = cls(
            connection_name=connection_name,
            schema_name=schema_name,
            directory_prefix=directory_prefix,
            run_checks_job_template=run_checks_job_template,
            run_profiling_checks_job_template=run_profiling_checks_job_template,
            run_monitoring_checks_job_template=run_monitoring_checks_job_template,
            run_partition_checks_job_template=run_partition_checks_job_template,
            collect_statistics_job_template=collect_statistics_job_template,
            import_table_job_parameters=import_table_job_parameters,
            data_clean_job_template=data_clean_job_template,
            can_edit=can_edit,
            can_collect_statistics=can_collect_statistics,
            can_run_checks=can_run_checks,
            can_delete_data=can_delete_data,
            error_message=error_message,
        )

        schema_model.additional_properties = d
        return schema_model

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class DqoJobType(str, Enum):
    AUTO_IMPORT_TABLES = "auto_import_tables"
    COLLECT_ERROR_SAMPLES = "collect_error_samples"
    COLLECT_ERROR_SAMPLES_ON_TABLE = "collect_error_samples_on_table"
    COLLECT_SCHEDULED_STATISTICS = "collect_scheduled_statistics"
    COLLECT_STATISTICS = "collect_statistics"
    COLLECT_STATISTICS_ON_TABLE = "collect_statistics_on_table"
    DELETE_STORED_DATA = "delete_stored_data"
    IMPORT_SCHEMA = "import_schema"
    IMPORT_TABLES = "import_tables"
    QUEUE_THREAD_SHUTDOWN = "queue_thread_shutdown"
    REPAIR_STORED_DATA = "repair_stored_data"
    RUN_CHECKS = "run_checks"
    RUN_CHECKS_ON_TABLE = "run_checks_on_table"
    RUN_SCHEDULED_CHECKS_CRON = "run_scheduled_checks_cron"
    SYNCHRONIZE_FOLDER = "synchronize_folder"
    SYNCHRONIZE_MULTIPLE_FOLDERS = "synchronize_multiple_folders"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class ProfilingTimePeriodTruncation(str, Enum):
    STORE_ALL_RESULTS_WITHOUT_DATE_TRUNCATION = (
        "store_all_results_without_date_truncation"
    )
    STORE_THE_MOST_RECENT_RESULT_PER_DAY = "store_the_most_recent_result_per_day"
    STORE_THE_MOST_RECENT_RESULT_PER_HOUR = "store_the_most_recent_result_per_hour"
    STORE_THE_MOST_RECENT_RESULT_PER_MONTH = "store_the_most_recent_result_per_month"
    STORE_THE_MOST_RECENT_RESULT_PER_WEEK = "store_the_most_recent_result_per_week"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.historic_data_points_grouping import HistoricDataPointsGrouping
from ..types import UNSET, Unset

T = TypeVar("T", bound="RuleTimeWindowSettingsSpec")


@_attrs_define
class RuleTimeWindowSettingsSpec:
    """
    Attributes:
        prediction_time_window (Union[Unset, int]): Number of historic time periods to look back for results. Returns
            results from previous time periods before the sensor readout timestamp to be used in a rule. Time periods are
            used in rules that need historic data to calculate an average to detect anomalies. e.g. when the sensor is
            configured to use a 'day' time period, the rule will receive results from the time_periods number of days before
            the time period in the sensor readout. The default is 14 (days).
        min_periods_with_readouts (Union[Unset, int]): Minimum number of past time periods with a sensor readout that
            must be present in the data in order to call the rule. The rule is not called and the sensor readout is
            discarded as not analyzable (not enough historic data to perform prediction) when the number of past sensor
            readouts is not met. The default is 7.
        historic_data_point_grouping (Union[Unset, HistoricDataPointsGrouping]):
    """

    prediction_time_window: Union[Unset, int] = UNSET
    min_periods_with_readouts: Union[Unset, int] = UNSET
    historic_data_point_grouping: Union[Unset, HistoricDataPointsGrouping] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        prediction_time_window = self.prediction_time_window
        min_periods_with_readouts = self.min_periods_with_readouts
        historic_data_point_grouping: Union[Unset, str] = UNSET
        if not isinstance(self.historic_data_point_grouping, Unset):
            historic_data_point_grouping = self.historic_data_point_grouping.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if prediction_time_window is not UNSET:
            field_dict["prediction_time_window"] = prediction_time_window
        if min_periods_with_readouts is not UNSET:
            field_dict["min_periods_with_readouts"] = min_periods_with_readouts
        if historic_data_point_grouping is not UNSET:
            field_dict["historic_data_point_grouping"] = historic_data_point_grouping

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        prediction_time_window = d.pop("prediction_time_window", UNSET)

        min_periods_with_readouts = d.pop("min_periods_with_readouts", UNSET)

        _historic_data_point_grouping = d.pop("historic_data_point_grouping", UNSET)
        historic_data_point_grouping: Union[Unset, HistoricDataPointsGrouping]
        if isinstance(_historic_data_point_grouping, Unset):
            historic_data_point_grouping = UNSET
        else:
            historic_data_point_grouping = HistoricDataPointsGrouping(
                _historic_data_point_grouping
            )

        rule_time_window_settings_spec = cls(
            prediction_time_window=prediction_time_window,
            min_periods_with_readouts=min_periods_with_readouts,
            historic_data_point_grouping=historic_data_point_grouping,
        )

        rule_time_window_settings_spec.additional_properties = d
        return rule_time_window_settings_spec

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="IcebergFileFormatSpec")


@_attrs_define
class IcebergFileFormatSpec:
    """
    Attributes:
        allow_moved_paths (Union[Unset, bool]): The option ensures that some path resolution is performed, which allows
            scanning Iceberg tables that are moved.
    """

    allow_moved_paths: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        allow_moved_paths = self.allow_moved_paths

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if allow_moved_paths is not UNSET:
            field_dict["allow_moved_paths"] = allow_moved_paths

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        allow_moved_paths = d.pop("allow_moved_paths", UNSET)

        iceberg_file_format_spec = cls(
            allow_moved_paths=allow_moved_paths,
        )

        iceberg_file_format_spec.additional_properties = d
        return iceberg_file_format_spec

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="HistogramDailyIssuesCount")


@_attrs_define
class HistogramDailyIssuesCount:
    """
    Attributes:
        warnings (Union[Unset, int]): The number of failed data quality checks that generated a warning severity data
            quality issue.
        errors (Union[Unset, int]): The number of failed data quality checks that generated an error severity data
            quality issue.
        fatals (Union[Unset, int]): The number of failed data quality checks that generated a fatal severity data
            quality issue.
        total_count (Union[Unset, int]): The total count of failed data quality checks on this day.
    """

    warnings: Union[Unset, int] = UNSET
    errors: Union[Unset, int] = UNSET
    fatals: Union[Unset, int] = UNSET
    total_count: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        warnings = self.warnings
        errors = self.errors
        fatals = self.fatals
        total_count = self.total_count

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if warnings is not UNSET:
            field_dict["warnings"] = warnings
        if errors is not UNSET:
            field_dict["errors"] = errors
        if fatals is not UNSET:
            field_dict["fatals"] = fatals
        if total_count is not UNSET:
            field_dict["totalCount"] = total_count

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        warnings = d.pop("warnings", UNSET)

        errors = d.pop("errors", UNSET)

        fatals = d.pop("fatals", UNSET)

        total_count = d.pop("totalCount", UNSET)

        histogram_daily_issues_count = cls(
            warnings=warnings,
            errors=errors,
            fatals=fatals,
            total_count=total_count,
        )

        histogram_daily_issues_count.additional_properties = d
        return histogram_daily_issues_count

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.table_duplicate_record_count_check_spec import (
        TableDuplicateRecordCountCheckSpec,
    )
    from ..models.table_duplicate_record_percent_check_spec import (
        TableDuplicateRecordPercentCheckSpec,
    )
    from ..models.table_uniqueness_daily_partition_checks_spec_custom_checks import (
        TableUniquenessDailyPartitionChecksSpecCustomChecks,
    )


T = TypeVar("T", bound="TableUniquenessDailyPartitionChecksSpec")


@_attrs_define
class TableUniquenessDailyPartitionChecksSpec:
    """
    Attributes:
        custom_checks (Union[Unset, TableUniquenessDailyPartitionChecksSpecCustomChecks]): Dictionary of additional
            custom checks within this category. The keys are check names defined in the definition section. The sensor
            parameters and rules should match the type of the configured sensor and rule for the custom check.
        daily_partition_duplicate_record_count (Union[Unset, TableDuplicateRecordCountCheckSpec]):
        daily_partition_duplicate_record_percent (Union[Unset, TableDuplicateRecordPercentCheckSpec]):
    """

    custom_checks: Union[
        Unset, "TableUniquenessDailyPartitionChecksSpecCustomChecks"
    ] = UNSET
    daily_partition_duplicate_record_count: Union[
        Unset, "TableDuplicateRecordCountCheckSpec"
    ] = UNSET
    daily_partition_duplicate_record_percent: Union[
        Unset, "TableDuplicateRecordPercentCheckSpec"
    ] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        custom_checks: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.custom_checks, Unset):
            custom_checks = self.custom_checks.to_dict()

        daily_partition_duplicate_record_count: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.daily_partition_duplicate_record_count, Unset):
            daily_partition_duplicate_record_count = (
                self.daily_partition_duplicate_record_count.to_dict()
            )

        daily_partition_duplicate_record_percent: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.daily_partition_duplicate_record_percent, Unset):
            daily_partition_duplicate_record_percent = (
                self.daily_partition_duplicate_record_percent.to_dict()
            )

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if custom_checks is not UNSET:
            field_dict["custom_checks"] = custom_checks
        if daily_partition_duplicate_record_count is not UNSET:
            field_dict["daily_partition_duplicate_record_count"] = (
                daily_partition_duplicate_record_count
            )
        if daily_partition_duplicate_record_percent is not UNSET:
            field_dict["daily_partition_duplicate_record_percent"] = (
                daily_partition_duplicate_record_percent
            )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.table_duplicate_record_count_check_spec import (
            TableDuplicateRecordCountCheckSpec,
        )
        from ..models.table_duplicate_record_percent_check_spec import (
            TableDuplicateRecordPercentCheckSpec,
        )
        from ..models.table_uniqueness_daily_partition_checks_spec_custom_checks import (
            TableUniquenessDailyPartitionChecksSpecCustomChecks,
        )

        d = src_dict.copy()
        _custom_checks = d.pop("custom_checks", UNSET)
        custom_checks: Union[Unset, TableUniquenessDailyPartitionChecksSpecCustomChecks]
        if isinstance(_custom_checks, Unset):
            custom_checks = UNSET
        else:
            custom_checks = (
                TableUniquenessDailyPartitionChecksSpecCustomChecks.from_dict(
                    _custom_checks
                )
            )

        _daily_partition_duplicate_record_count = d.pop(
            "daily_partition_duplicate_record_count", UNSET
        )
        daily_partition_duplicate_record_count: Union[
            Unset, TableDuplicateRecordCountCheckSpec
        ]
        if isinstance(_daily_partition_duplicate_record_count, Unset):
            daily_partition_duplicate_record_count = UNSET
        else:
            daily_partition_duplicate_record_count = (
                TableDuplicateRecordCountCheckSpec.from_dict(
                    _daily_partition_duplicate_record_count
                )
            )

        _daily_partition_duplicate_record_percent = d.pop(
            "daily_partition_duplicate_record_percent", UNSET
        )
        daily_partition_duplicate_record_percent: Union[
            Unset, TableDuplicateRecordPercentCheckSpec
        ]
        if isinstance(_daily_partition_duplicate_record_percent, Unset):
            daily_partition_duplicate_record_percent = UNSET
        else:
            daily_partition_duplicate_record_percent = (
                TableDuplicateRecordPercentCheckSpec.from_dict(
                    _daily_partition_duplicate_record_percent
                )
            )

        table_uniqueness_daily_partition_checks_spec = cls(
            custom_checks=custom_checks,
            daily_partition_duplicate_record_count=daily_partition_duplicate_record_count,
            daily_partition_duplicate_record_percent=daily_partition_duplicate_record_percent,
        )

        table_uniqueness_daily_partition_checks_spec.additional_properties = d
        return table_uniqueness_daily_partition_checks_spec

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.dqo_job_status import DqoJobStatus
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.dqo_queue_job_id import DqoQueueJobId
    from ..models.run_checks_result import RunChecksResult


T = TypeVar("T", bound="RunChecksQueueJobResult")


@_attrs_define
class RunChecksQueueJobResult:
    """Object returned from the operation that queues a "run checks" job. The result contains the job id that was started
    and optionally can also contain the result of running the checks if the operation was started with wait=true
    parameter to wait for the "run checks" job to finish.

        Attributes:
            job_id (Union[Unset, DqoQueueJobId]): Identifies a single job that was pushed to the job queue.
            result (Union[Unset, RunChecksResult]): Returns the result (highest data quality check severity and the finished
                checks count) for the checks that were recently executed.
            status (Union[Unset, DqoJobStatus]):
    """

    job_id: Union[Unset, "DqoQueueJobId"] = UNSET
    result: Union[Unset, "RunChecksResult"] = UNSET
    status: Union[Unset, DqoJobStatus] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        job_id: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.job_id, Unset):
            job_id = self.job_id.to_dict()

        result: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.result, Unset):
            result = self.result.to_dict()

        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if job_id is not UNSET:
            field_dict["jobId"] = job_id
        if result is not UNSET:
            field_dict["result"] = result
        if status is not UNSET:
            field_dict["status"] = status

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.dqo_queue_job_id import DqoQueueJobId
        from ..models.run_checks_result import RunChecksResult

        d = src_dict.copy()
        _job_id = d.pop("jobId", UNSET)
        job_id: Union[Unset, DqoQueueJobId]
        if isinstance(_job_id, Unset):
            job_id = UNSET
        else:
            job_id = DqoQueueJobId.from_dict(_job_id)

        _result = d.pop("result", UNSET)
        result: Union[Unset, RunChecksResult]
        if isinstance(_result, Unset):
            result = UNSET
        else:
            result = RunChecksResult.from_dict(_result)

        _status = d.pop("status", UNSET)
        status: Union[Unset, DqoJobStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = DqoJobStatus(_status)

        run_checks_queue_job_result = cls(
            job_id=job_id,
            result=result,
            status=status,
        )

        run_checks_queue_job_result.additional_properties = d
        return run_checks_queue_job_result

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.rule_runner_type import RuleRunnerType
from ..models.rule_time_window_mode import RuleTimeWindowMode
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.parameter_definition_spec import ParameterDefinitionSpec
    from ..models.rule_model_parameters import RuleModelParameters
    from ..models.rule_time_window_settings_spec import RuleTimeWindowSettingsSpec


T = TypeVar("T", bound="RuleModel")


@_attrs_define
class RuleModel:
    """Rule model

    Attributes:
        rule_name (Union[Unset, str]): Rule name
        rule_python_module_content (Union[Unset, str]): Rule Python module content
        type (Union[Unset, RuleRunnerType]):
        java_class_name (Union[Unset, str]): Java class name for a rule runner that will execute the sensor. The "type"
            must be "java_class".
        mode (Union[Unset, RuleTimeWindowMode]):
        time_window (Union[Unset, RuleTimeWindowSettingsSpec]):
        fields (Union[Unset, List['ParameterDefinitionSpec']]): List of fields that are parameters of a custom rule.
            Those fields are used by the DQOps UI to display the data quality check editing screens with proper UI controls
            for all required fields.
        parameters (Union[Unset, RuleModelParameters]): Additional rule parameters
        custom (Union[Unset, bool]): This rule has a custom (user level) definition.
        built_in (Union[Unset, bool]): This rule has is a built-in rule.
        can_edit (Union[Unset, bool]): Boolean flag that decides if the current user can update or delete this object.
        yaml_parsing_error (Union[Unset, str]): Optional parsing error that was captured when parsing the YAML file.
            This field is null when the YAML file is valid. If an error was captured, this field returns the file parsing
            error message and the file location.
    """

    rule_name: Union[Unset, str] = UNSET
    rule_python_module_content: Union[Unset, str] = UNSET
    type: Union[Unset, RuleRunnerType] = UNSET
    java_class_name: Union[Unset, str] = UNSET
    mode: Union[Unset, RuleTimeWindowMode] = UNSET
    time_window: Union[Unset, "RuleTimeWindowSettingsSpec"] = UNSET
    fields: Union[Unset, List["ParameterDefinitionSpec"]] = UNSET
    parameters: Union[Unset, "RuleModelParameters"] = UNSET
    custom: Union[Unset, bool] = UNSET
    built_in: Union[Unset, bool] = UNSET
    can_edit: Union[Unset, bool] = UNSET
    yaml_parsing_error: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        rule_name = self.rule_name
        rule_python_module_content = self.rule_python_module_content
        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        java_class_name = self.java_class_name
        mode: Union[Unset, str] = UNSET
        if not isinstance(self.mode, Unset):
            mode = self.mode.value

        time_window: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.time_window, Unset):
            time_window = self.time_window.to_dict()

        fields: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.fields, Unset):
            fields = []
            for fields_item_data in self.fields:
                fields_item = fields_item_data.to_dict()

                fields.append(fields_item)

        parameters: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.parameters, Unset):
            parameters = self.parameters.to_dict()

        custom = self.custom
        built_in = self.built_in
        can_edit = self.can_edit
        yaml_parsing_error = self.yaml_parsing_error

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if rule_name is not UNSET:
            field_dict["rule_name"] = rule_name
        if rule_python_module_content is not UNSET:
            field_dict["rule_python_module_content"] = rule_python_module_content
        if type is not UNSET:
            field_dict["type"] = type
        if java_class_name is not UNSET:
            field_dict["java_class_name"] = java_class_name
        if mode is not UNSET:
            field_dict["mode"] = mode
        if time_window is not UNSET:
            field_dict["time_window"] = time_window
        if fields is not UNSET:
            field_dict["fields"] = fields
        if parameters is not UNSET:
            field_dict["parameters"] = parameters
        if custom is not UNSET:
            field_dict["custom"] = custom
        if built_in is not UNSET:
            field_dict["built_in"] = built_in
        if can_edit is not UNSET:
            field_dict["can_edit"] = can_edit
        if yaml_parsing_error is not UNSET:
            field_dict["yaml_parsing_error"] = yaml_parsing_error

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.parameter_definition_spec import ParameterDefinitionSpec
        from ..models.rule_model_parameters import RuleModelParameters
        from ..models.rule_time_window_settings_spec import RuleTimeWindowSettingsSpec

        d = src_dict.copy()
        rule_name = d.pop("rule_name", UNSET)

        rule_python_module_content = d.pop("rule_python_module_content", UNSET)

        _type = d.pop("type", UNSET)
        type: Union[Unset, RuleRunnerType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = RuleRunnerType(_type)

        java_class_name = d.pop("java_class_name", UNSET)

        _mode = d.pop("mode", UNSET)
        mode: Union[Unset, RuleTimeWindowMode]
        if isinstance(_mode, Unset):
            mode = UNSET
        else:
            mode = RuleTimeWindowMode(_mode)

        _time_window = d.pop("time_window", UNSET)
        time_window: Union[Unset, RuleTimeWindowSettingsSpec]
        if isinstance(_time_window, Unset):
            time_window = UNSET
        else:
            time_window = RuleTimeWindowSettingsSpec.from_dict(_time_window)

        fields = []
        _fields = d.pop("fields", UNSET)
        for fields_item_data in _fields or []:
            fields_item = ParameterDefinitionSpec.from_dict(fields_item_data)

            fields.append(fields_item)

        _parameters = d.pop("parameters", UNSET)
        parameters: Union[Unset, RuleModelParameters]
        if isinstance(_parameters, Unset):
            parameters = UNSET
        else:
            parameters = RuleModelParameters.from_dict(_parameters)

        custom = d.pop("custom", UNSET)

        built_in = d.pop("built_in", UNSET)

        can_edit = d.pop("can_edit", UNSET)

        yaml_parsing_error = d.pop("yaml_parsing_error", UNSET)

        rule_model = cls(
            rule_name=rule_name,
            rule_python_module_content=rule_python_module_content,
            type=type,
            java_class_name=java_class_name,
            mode=mode,
            time_window=time_window,
            fields=fields,
            parameters=parameters,
            custom=custom,
            built_in=built_in,
            can_edit=can_edit,
            yaml_parsing_error=yaml_parsing_error,
        )

        rule_model.additional_properties = d
        return rule_model

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.check_list_model import CheckListModel


T = TypeVar("T", bound="CheckContainerListModel")


@_attrs_define
class CheckContainerListModel:
    """Simplistic model that returns the list of data quality checks, their names, categories and "configured" flag.

    Attributes:
        checks (Union[Unset, List['CheckListModel']]): Simplistic list of all data quality checks.
        can_edit (Union[Unset, bool]): Boolean flag that decides if the current user can edit the check.
        can_run_checks (Union[Unset, bool]): Boolean flag that decides if the current user can run checks.
        can_delete_data (Union[Unset, bool]): Boolean flag that decides if the current user can delete data (results).
    """

    checks: Union[Unset, List["CheckListModel"]] = UNSET
    can_edit: Union[Unset, bool] = UNSET
    can_run_checks: Union[Unset, bool] = UNSET
    can_delete_data: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        checks: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.checks, Unset):
            checks = []
            for checks_item_data in self.checks:
                checks_item = checks_item_data.to_dict()

                checks.append(checks_item)

        can_edit = self.can_edit
        can_run_checks = self.can_run_checks
        can_delete_data = self.can_delete_data

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if checks is not UNSET:
            field_dict["checks"] = checks
        if can_edit is not UNSET:
            field_dict["can_edit"] = can_edit
        if can_run_checks is not UNSET:
            field_dict["can_run_checks"] = can_run_checks
        if can_delete_data is not UNSET:
            field_dict["can_delete_data"] = can_delete_data

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.check_list_model import CheckListModel

        d = src_dict.copy()
        checks = []
        _checks = d.pop("checks", UNSET)
        for checks_item_data in _checks or []:
            checks_item = CheckListModel.from_dict(checks_item_data)

            checks.append(checks_item)

        can_edit = d.pop("can_edit", UNSET)

        can_run_checks = d.pop("can_run_checks", UNSET)

        can_delete_data = d.pop("can_delete_data", UNSET)

        check_container_list_model = cls(
            checks=checks,
            can_edit=can_edit,
            can_run_checks=can_run_checks,
            can_delete_data=can_delete_data,
        )

        check_container_list_model.additional_properties = d
        return check_container_list_model

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
